from contextlib import ExitStack

from matplotlib.axes import Axes
import matplotlib.transforms as transforms
import matplotlib.axis as maxis
import matplotlib.spines as mspines
from matplotlib.projections import register_projection


# The sole purpose of this class is to look at the upper, lower, or total
# interval as appropriate and see what parts of the tick to draw, if any.
class SkewXTick(maxis.XTick):
    def draw(self, renderer):
        # When adding the callbacks with `stack.callback`, we fetch the current
        # visibility state of the artist with `get_visible`; the ExitStack will
        # restore these states (`set_visible`) at the end of the block (after
        # the draw).
        with ExitStack() as stack:
            for artist in [self.gridline, self.tick1line, self.tick2line,
                           self.label1, self.label2]:
                stack.callback(artist.set_visible, artist.get_visible())
            needs_lower = transforms.interval_contains(
                self.axes.lower_xlim, self.get_loc())
            needs_upper = transforms.interval_contains(
                self.axes.upper_xlim, self.get_loc())
            self.tick1line.set_visible(
                self.tick1line.get_visible() and needs_lower)
            self.label1.set_visible(
                self.label1.get_visible() and needs_lower)
            self.tick2line.set_visible(
                self.tick2line.get_visible() and needs_upper)
            self.label2.set_visible(
                self.label2.get_visible() and needs_upper)
            super(SkewXTick, self).draw(renderer)

    def get_view_interval(self):
        return self.axes.xaxis.get_view_interval()


# This class exists to provide two separate sets of intervals to the tick,
# as well as create instances of the custom tick
class SkewXAxis(maxis.XAxis):
    def _get_tick(self, major):
        return SkewXTick(self.axes, None, major=major)

    def get_view_interval(self):
        return self.axes.upper_xlim[0], self.axes.lower_xlim[1]


# This class exists to calculate the separate data range of the
# upper X-axis and draw the spine there. It also provides this range
# to the X-axis artist for ticking and gridlines
class SkewSpine(mspines.Spine):
    def _adjust_location(self):
        pts = self._path.vertices
        if self.spine_type == 'top':
            pts[:, 0] = self.axes.upper_xlim
        else:
            pts[:, 0] = self.axes.lower_xlim


# This class handles registration of the skew-xaxes as a projection as well
# as setting up the appropriate transformations. It also overrides standard
# spines and axes instances as appropriate.
class SkewXAxes(Axes):
    # The projection must specify a name.  This will be used be the
    # user to select the projection, i.e. ``subplot(111,
    # projection='skewx')``.
    name = 'skewx'

    def _init_axis(self):
        # Taken from Axes and modified to use our modified X-axis
        self.xaxis = SkewXAxis(self)
        self.spines['top'].register_axis(self.xaxis)
        self.spines['bottom'].register_axis(self.xaxis)
        self.yaxis = maxis.YAxis(self)
        self.spines['left'].register_axis(self.yaxis)
        self.spines['right'].register_axis(self.yaxis)

    def _gen_axes_spines(self):
        spines = {'top': SkewSpine.linear_spine(self, 'top'),
                  'bottom': mspines.Spine.linear_spine(self, 'bottom'),
                  'left': mspines.Spine.linear_spine(self, 'left'),
                  'right': mspines.Spine.linear_spine(self, 'right')}
        return spines

    def _set_lim_and_transforms(self):
        """
        This is called once when the plot is created to set up all the
        transforms for the data, text and grids.
        """
        rot = 30

        # Get the standard transform setup from the Axes base class
        super()._set_lim_and_transforms()

        # Need to put the skew in the middle, after the scale and limits,
        # but before the transAxes. This way, the skew is done in Axes
        # coordinates thus performing the transform around the proper origin
        # We keep the pre-transAxes transform around for other users, like the
        # spines for finding bounds
        self.transDataToAxes = (
            self.transScale
            + self.transLimits
            + transforms.Affine2D().skew_deg(rot, 0)
        )
        # Create the full transform from Data to Pixels
        self.transData = self.transDataToAxes + self.transAxes

        # Blended transforms like this need to have the skewing applied using
        # both axes, in axes coords like before.
        self._xaxis_transform = (
            transforms.blended_transform_factory(
                self.transScale + self.transLimits,
                transforms.IdentityTransform())
            + transforms.Affine2D().skew_deg(rot, 0)
            + self.transAxes
        )

    @property
    def lower_xlim(self):
        return self.axes.viewLim.intervalx

    @property
    def upper_xlim(self):
        pts = [[0., 1.], [1., 1.]]
        return self.transDataToAxes.inverted().transform(pts)[:, 0]





import numpy as np
import netCDF4 as nc
import os
import datetime as dt
import bz2
import time
from dateutil.relativedelta import relativedelta
import pandas as pd
import urllib

# Author:hujunnan

np.set_printoptions(suppress=True)


def pad2D(data, col):
    return np.asarray(list(map(lambda x: x + ([np.nan] * (col - len(x))), data)))


def pad3D(data, radialMax, distance):
    tmp = list(map(lambda x: x + ([[np.nan] * distance] * (radialMax - len(x))), data))
    for i in range(len(tmp)):
        tmp[i] = pad2D(tmp[i], distance)
    return np.asarray(tmp)


class readCinrad():
    def __init__(self, path):
        self.path=path
        self.datetime = 0
        self.radarID=path.split("_")[3]
        self.type = path.split("_")[7]
        if ".bz2" in path:
            self.unzipBZ2()
        self.fi = open(self.path, "rb")
        self.Cut_Number = 0
        self.Log_Resolution = 0
        self.Doppler_Resolution = 0
        self.isDuelDict = {
            True: "双",
            False: "单"
        }
        self.isDual=False

    def seconds2datetime(self,seconds):
        return dt.datetime.strptime(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(seconds)), "%Y-%m-%d %H:%M:%S")

    def unzipBZ2(self):
        f = open(file=self.path, mode='rb')
        un = f.read()
        f.close()
        username = bz2.decompress(un)
        os.makedirs("./tmp",exist_ok=True)
        self.path=os.path.join("./tmp",os.path.basename(self.path).replace(".bz2",""))
        with open(self.path, 'wb')as fp:
            fp.write(username)

    def parse_GENERIC_HEADER(self):
        # 站点参数
        GENERIC_HEADER = np.dtype(
            {'names': ['Magic_Number', 'Major_Version', 'Minor_Version', 'Generic_Type', 'Product_Type', 'reserve'],
             'formats': ['i', 'h', 'h', 'i', 'i', 'S16']})
        [Magic_Number, Major_Version, Minor_Version, Generic_Type, Product_Type, _] = \
            np.fromfile(file=self.fi, dtype=GENERIC_HEADER, count=1, sep='')[0]
        print(Magic_Number, Major_Version, Minor_Version, Generic_Type, Product_Type)

    def parse_SITE_CONFIG(self):
        # 站点参数
        SITE_CONFIG = np.dtype(
            {'names': ['Site_Code', 'Site_Name', 'Latitude', 'Longitude', 'Antenna_Height', 'Ground_Height',
                       'Frequency', 'Beam_Width_Hori', 'Beam_Width_Vert', 'RDA_Version', 'Radar_Type', 'reserve'],
             'formats': ['S8', 'S32', 'f', 'f', 'i', 'i', 'f', 'f', 'f', 'i', 'h', 'S54']})
        [Site_Code, Site_Name, Latitude, Longitude, Antenna_Height, Ground_Height, Frequency, Beam_Width_Hori,
         Beam_Width_Vert, RDA_Version, Radar_Type, _] = np.fromfile(file=self.fi, dtype=SITE_CONFIG, count=1, sep='')[0]
        print(Site_Code, Site_Name, Latitude, Longitude, Antenna_Height, Ground_Height, Frequency, Beam_Width_Hori,
              Beam_Width_Vert, RDA_Version, Radar_Type)

    def parse_TASK_CONFIG(self):
        # //TASK CONFIG
        TASK_CONFIG = np.dtype(
            {'names': ['Task_Name', ' Task_Description', 'Polarization_Type', 'Scan_Type', 'Pulse_Width',
                       'Scan_Start_Time', 'Cut_Number', 'Horizontal_Noise', 'Vertical_Noise', 'Horizontal_Calibration',
                       'Vertical_Calibration', 'Horizontal_Noise_Temperature', 'Vertical_Noise_Temperature',
                       'ZDR_Calibration',
                       'PHIDP_Calibration', 'LDR_Calibration', 'reserve'],
             'formats': ['S32', 'S128', 'i', 'i', 'i', 'i', 'i', 'f', 'f', 'f', 'f', 'f', 'f', 'f', 'f', 'f', 'S40']})
        [Task_Name, Task_Description, Polarization_Type, Scan_Type, Pulse_Width, Scan_Start_Time, Cut_Number,
         Horizontal_Noise, Vertical_Noise, Horizontal_Calibration, Vertical_Calibration, Horizontal_Noise_Temperature,
         Vertical_Noise_Temperature, ZDR_Calibration, PHIDP_Calibration, LDR_Calibration, _] = \
            np.fromfile(file=self.fi, dtype=TASK_CONFIG, count=1, sep='')[0]

        self.datetime=self.seconds2datetime(Scan_Start_Time)
        # //系统LDR标定偏差
        # in.readStringHJN(40)
        # //保留字段
        #
        self.Cut_Number = Cut_Number
        if "dual" in str(Task_Description):
            self.isDual = True
        else:
            self.isDual = False
        print(Task_Name, Task_Description)
        print(Polarization_Type, Scan_Type, Pulse_Width, Scan_Start_Time, Cut_Number, Horizontal_Noise, Vertical_Noise,
              Horizontal_Calibration,
              Vertical_Calibration, Horizontal_Noise_Temperature, Vertical_Noise_Temperature, ZDR_Calibration,
              PHIDP_Calibration, LDR_Calibration)

        print("%s偏振" % (self.isDuelDict[self.isDual]))

    def parse_read_CUT_CONFIG(self):
        CUT_CONFIG1 = np.dtype(
            {'names': ['Process_Mode', 'Wave_Form', 'PRF_1', 'PRF_2', 'Dealiasing_Mode', 'Azimuth',
                       'Elevation', 'Start_Angle', 'End_Angle', 'Angular_Resolution', 'Scan_Speed',
                       'Log_Resolution', 'Doppler_Resolution', 'Maximum_Range1', 'Maximum_Range2', 'Start_Range',
                       'Sample_1', 'Sample_2', 'Phase_Mode', 'Atmospheric_Loss', 'Nyquist_Speed'],
             'formats': ['i', 'i', 'f', 'f', 'i', 'f', 'f', 'f', 'f', 'f', 'f', 'i', 'i', 'i', 'i', 'i', 'i',
                         'i', 'i', 'f', 'f']})
        [Process_Mode, Wave_Form, PRF_1, PRF_2, Dealiasing_Mode, Azimuth, Elevation, Start_Angle,
         End_Angle, Angular_Resolution, Scan_Speed, Log_Resolution, Doppler_Resolution, Maximum_Range1, Maximum_Range2,
         Start_Range, Sample_1, Sample_2, Phase_Mode, Atmospheric_Loss, Nyquist_Speed] = \
            np.fromfile(file=self.fi, dtype=CUT_CONFIG1, count=1, sep='')[0]
        print("扫描字块 Process_Mode:", Process_Mode, "Wave_Form:", Wave_Form, "PRF_1:", PRF_1, "PRF_2:", PRF_2,
              "Dealiasing_Mode:", Dealiasing_Mode, "Azimuth:", Azimuth, "Elevation:", Elevation, "Start_Angle:",
              Start_Angle, "End_Angle:", End_Angle, "Angular_Resolution:", Angular_Resolution, "Scan_Speed:",
              Scan_Speed, "Log_Resolution:", Log_Resolution, "Doppler_Resolution:", Doppler_Resolution,
              "Maximum_Range1:",
              Maximum_Range1, "Maximum_Range2:", Maximum_Range2, "Start_Range:", Start_Range, "Sample_1:", Sample_1,
              "Sample_2:", Sample_2, "Phase_Mode:", Phase_Mode, "Atmospheric_Loss:", Atmospheric_Loss, "Nyquist_Speed:",
              Nyquist_Speed)

        self.Log_Resolution = Log_Resolution
        self.Doppler_Resolution = Doppler_Resolution
        CUT_CONFIG2 = np.dtype(
            {'names': ['Moments_Mask', 'Moments_Size_Mask', 'Misc_Filter_Mask', 'SQI_Threshold', 'SIG_Threshold',
                       'CSR_Threshold',
                       'LOG_Threshold', 'CPA_Threshold', 'PMI_Threshold', 'DPLOG_Threshold',
                       'Thresholds_r', 'dBT_Mask', 'dBZ_Mask', 'Velocity_Mask', 'Spectrum_Width_Mask', 'DP_Mask',
                       'Mask_Reserved', 'Scan_Sync',
                       'Direction'],
             'formats': ['q', 'q', 'i', 'f', 'f', 'f', 'f', 'f', 'f', 'f', 'S4', 'i', 'i', 'i', 'i', 'i',
                         'S12', 'i', 'i']})
        [Moments_Mask, Moments_Size_Mask, Misc_Filter_Mask, SQI_Threshold, SIG_Threshold, CSR_Threshold,
         LOG_Threshold, CPA_Threshold, PMI_Threshold, DPLOG_Threshold,
         Thresholds_r, dBT_Mask, dBZ_Mask, Velocity_Mask, Spectrum_Width_Mask, DP_Mask, Mask_Reserved, Scan_Sync,
         Direction] = \
            np.fromfile(file=self.fi, dtype=CUT_CONFIG2, count=1, sep='')[0]

        print(Moments_Mask, Moments_Size_Mask, Misc_Filter_Mask, SQI_Threshold, SIG_Threshold, CSR_Threshold,
              LOG_Threshold, CPA_Threshold, PMI_Threshold, DPLOG_Threshold,
              Thresholds_r, dBT_Mask, dBZ_Mask, Velocity_Mask, Spectrum_Width_Mask, DP_Mask, Mask_Reserved, Scan_Sync,
              Direction)

        CUT_CONFIG3 = np.dtype(
            {'names': ['Ground_Clutter_Classifier_Type', 'Ground_Clutter_Filter_Type',
                       'Ground_Clutter_Filter_Notch_Width',
                       'Ground_Clutter_Filter_Window', "reserve"],
             'formats': ['h', 'h', 'h', 'h', 'S72']})
        [Ground_Clutter_Classifier_Type, Ground_Clutter_Filter_Type, Ground_Clutter_Filter_Notch_Width,
         Ground_Clutter_Filter_Window, _] = \
            np.fromfile(file=self.fi, dtype=CUT_CONFIG3, count=1, sep='')[0]
        print(Ground_Clutter_Classifier_Type, Ground_Clutter_Filter_Type, Ground_Clutter_Filter_Notch_Width,
              Ground_Clutter_Filter_Window)

    def parse_MOMENT_DATA(self, eleNum):

        rData = []
        vData = []
        sData = []
        tData = []
        SQIData = []
        ZDRData = []
        CCData = []
        DPData = []
        KDPData = []
        SNRHData = []
        azimuthR = []
        azimuthV = []
        elevationR = []
        elevationV = []
        numGatesR = 0
        numGatesV = 0
        numRadialsR = 0
        numRadialsV = 0
        timeR = []
        timeV = []

        # /**
        #   * 径向数据状态
        #   * 0–仰角开始
        #   * 1–中间数据
        #   * 2–仰角结束
        #   * 3–体扫开始
        #   * 4–体扫结束
        #   * 5–RHI开始
        #   * 6–RHI结束
        #   */
        isFinished = 0
        Length_of_data_all = 0

        while not (isFinished == 2 or isFinished == 4):

            MOMENT_DATA_BLOCK_HEAD = np.dtype(
                {'names': ['Radial_State', 'Spot_Blank', 'Sequence_Number', 'Radial_Number', 'Elevation_Number',
                           'Azimuth',
                           'Elevation', 'Seconds', 'Microseconds', 'Length_of_data', 'Moment_Number', "reserve"],
                 'formats': ['i', 'i', 'i', 'i', 'i', 'f', 'f', 'i', 'i', 'i', 'i', 'S20']})
            [Radial_State, Spot_Blank, Sequence_Number, Radial_Number, Elevation_Number, Azimuth, Elevation, Seconds,
             Microseconds
                , Length_of_data, Moment_Number, _] = \
                np.fromfile(file=self.fi, dtype=MOMENT_DATA_BLOCK_HEAD, count=1, sep='')[0]
            isFinished = Radial_State
            # print("径向数据块头 Radial_State:", Radial_State, "Spot_Blank:", Spot_Blank, "Sequence_Number:", Sequence_Number,
            #       "Radial_Number:", Radial_Number, "Elevation_Number:", Elevation_Number, "Azimuth3:", Azimuth,
            #       "Elevation:", Elevation, "Seconds:", Seconds, "Microseconds:", Microseconds, "Length_of_data:",
            #       Length_of_data, "Moment_Number:", Moment_Number)

            if Elevation_Number != eleNum:
                print("something wrong")
                return None

            azimuthR.append(Azimuth)
            azimuthV.append(Azimuth)

            elevationR.append(Elevation)
            elevationV.append(Elevation)

            timeR.append(Seconds)
            timeV.append(Seconds)

            for i in range(Moment_Number):
                MOMENT_DATA_HEAD = np.dtype(
                    {'names': ['Data_Type', 'Scale', 'Offset', 'Bin_Length', 'Flags', 'Length', 'reserve'],
                     'formats': ['i', 'i', 'i', 'h', 'h', 'i', 'S12']})
                [Data_Type, Scale, Offset, Bin_Length, Flags, Length, _] = \
                    np.fromfile(file=self.fi, dtype=MOMENT_DATA_HEAD, count=1, sep='')[0]

                # print("   径向数据头 Data_Type:", Data_Type, "Scale:", Scale, "Offset:", Offset, "Bin_Length:", Bin_Length, "Flags:",
                #       Flags, "Length:", Length)

                dtypeType = None
                if Bin_Length == 1:
                    dtypeType = np.uint8
                else:
                    dtypeType = np.uint16
                data = np.fromfile(file=self.fi, dtype=dtypeType, count=Length // Bin_Length)
                data = data.astype(np.float32)
                data[data < 5] = np.nan
                data = list((data - Offset) / Scale)

                # //添加数据块
                if Data_Type == 2:
                    rData.append(data)
                    numGatesR = Length / Bin_Length
                    numRadialsR = Radial_Number
                # print("lengthR:", numGatesR, numGatesV)

                elif (Data_Type == 3):
                    vData.append(data)
                    numGatesV = Length / Bin_Length
                    numRadialsV = Radial_Number
                # print("lengthV:", numGatesR, numGatesV)

                elif (Data_Type == 4):
                    sData.append(data)

                elif (Data_Type == 1):
                    tData.append(data)
                elif (Data_Type == 5):
                    SQIData.append(data)
                elif (Data_Type == 7) :
                    ZDRData.append(data)
                elif (Data_Type == 9) :
                    CCData.append(data)
                elif (Data_Type == 10) :
                    DPData.append(data)
                elif (Data_Type == 11) :
                    KDPData.append(data)
                elif (Data_Type == 16):
                    SNRHData.append(data)


        return {"Length_of_data_all": Length_of_data_all, "azimuthR": azimuthR, "azimuthV": azimuthV,
                "elevationR": elevationR, "elevationV": elevationV, "numGatesR": numGatesR, "numGatesV": numGatesV,
                "numRadialsR": numRadialsR, "numRadialsV": numRadialsV, "timeR": timeR, "timeV": timeV,
                "RadialVelocity": vData, "Reflectivity": rData, "SpectrumWidth": sData, "TotalReflectivity": tData,
                "SignalQualityIndex": SQIData, "DifferentialReflectivity": ZDRData,
                "CrossCorrelationCoefficient": CCData, "DifferentialPhase": DPData,
                "SpecificDifferentialPhase": KDPData, "HorizontalSignalNoiseRatio": SNRHData}

    def mkNC(self, outpath, rC):
        os.makedirs(os.path.dirname(outpath), exist_ok=True)
        a = nc.Dataset(outpath, "w", format='NETCDF4')
        a.createDimension("scanR", len(rC["elevationR"]))
        a.createDimension("scanV", len(rC["elevationV"]))
        radialMax = max(np.max(rC["numRadialsR"]), np.max(rC["numRadialsV"]))

        # println(radialMax)
        a.createDimension("radial", radialMax)
        a.createDimension("gateR", len(rC["distanceR"]))
        a.createDimension("gateV", len(rC["distanceV"]))

        a.createVariable("azimuthR", "f", ["scanR", "radial"])
        a.createVariable("azimuthV", "f", ["scanV", "radial"])
        a.createVariable("distanceR", "f", ["gateR"])
        a.createVariable("distanceV", "f", ["gateV"])
        a.createVariable("elevationR", "f", ["scanR", "radial"])
        a.createVariable("elevationV", "f", ["scanV", "radial"])
        a.createVariable("numGatesR", "i", ["scanR"])
        a.createVariable("numGatesV", "i", ["scanV"])
        a.createVariable("numRadialsR", "i", ["scanR"])
        a.createVariable("numRadialsV", "i", ["scanV"])
        tR = a.createVariable("timeR", "i", ["scanR", "radial"])
        tR.missing_value = -1
        tV = a.createVariable("timeV", "i", ["scanV", "radial"])
        tV.missing_value = -1
        v = a.createVariable("RadialVelocity", "f", ["scanV", "radial", "gateV"], zlib=True)
        v.valid_range = [-62, 62]
        a.createVariable("SpectrumWidth", "f", ["scanV", "radial", "gateV"], zlib=True)
        a.createVariable("Reflectivity", "f", ["scanR", "radial", "gateR"], zlib=True)
        if self.isDual:
            a.createVariable("TotalReflectivity", "f", ["scanR", "radial", "gateR"], zlib=True)
            a.createVariable("SignalQualityIndex", "f", ["radial", "gateR"], zlib=True)
            a.createVariable("DifferentialReflectivity", "f", ["scanR", "radial", "gateR"], zlib=True)
            a.createVariable("CrossCorrelationCoefficient", "f", ["scanR", "radial", "gateR"], zlib=True)
            a.createVariable("DifferentialPhase", "f", ["scanR", "radial", "gateR"], zlib=True)
            a.createVariable("SpecificDifferentialPhase", "f4", ["scanR", "radial", "gateR"], zlib=True)
            a.createVariable("HorizontalSignalNoiseRatio", "f4", ["scanR", "radial", "gateR"], zlib=True)

        # print(len(rC["azimuthR"]), len(rC["azimuthR"][0]))
        # print(rC["azimuthR"])
        # print(a.variables["azimuthR"][:].shape)
        # print(rC["azimuthR"].shape)
        a.variables["azimuthR"][:] = rC["azimuthR"]
        a.variables["azimuthV"][:] = rC["azimuthV"]
        a.variables["distanceR"][:] = rC["distanceR"]
        a.variables["distanceV"][:] = rC["distanceV"]
        a.variables["elevationR"][:] = rC["elevationR"]
        a.variables["elevationV"][:] = rC["elevationV"]
        a.variables["numGatesR"][:] = rC["numGatesR"]
        a.variables["numGatesV"][:] = rC["numGatesV"]
        a.variables["numRadialsR"][:] = rC["numRadialsR"]
        a.variables["numRadialsV"][:] = rC["numRadialsV"]
        a.variables["timeR"][:] = rC["timeR"]
        a.variables["timeV"][:] = rC["timeV"]
        a.variables["RadialVelocity"][:] = rC["RadialVelocity"]
        a.variables["SpectrumWidth"][:] = rC["SpectrumWidth"]
        a.variables["Reflectivity"][:] = rC["Reflectivity"]
        if self.isDual:
            a.variables["TotalReflectivity"][:]=rC["TotalReflectivity"]
            a.variables["SignalQualityIndex"][:]=rC["SignalQualityIndex"]
            a.variables["DifferentialReflectivity"][:]=rC["DifferentialReflectivity"]
            a.variables["CrossCorrelationCoefficient"][:]=rC["CrossCorrelationCoefficient"]
            a.variables["DifferentialPhase"][:]=rC["DifferentialPhase"]
            a.variables["SpecificDifferentialPhase"][:]=rC["SpecificDifferentialPhase"]
            a.variables["HorizontalSignalNoiseRatio"][:]=rC["HorizontalSignalNoiseRatio"]

        a.close()

        return None

    def main(self,outparent):
        self.parse_GENERIC_HEADER()
        self.parse_SITE_CONFIG()
        self.parse_TASK_CONFIG()
        for i in range(self.Cut_Number):
            self.parse_read_CUT_CONFIG()

        rData = []
        vData = []
        sData = []
        azimuthR = []
        azimuthV = []
        tData = []
        SQIData = []
        SQIData2D = []
        ZDRData = []
        CCData = []
        DPData = []
        KDPData = []
        SNRHData = []
        distanceR = []
        distanceV = []
        elevationR = []
        elevationV = []
        numGatesR = []
        numGatesV = []
        numRadialsR = []
        numRadialsV = []
        timeR = []
        timeV = []

        for i in range(self.Cut_Number):
            a = self.parse_MOMENT_DATA(i + 1)

            if len(a["Reflectivity"]) != 0:
                azimuthR.append(a["azimuthR"])
                elevationR.append(a["elevationR"])

                # // println(a.Reflectivity.length)
                # // println(s"a.numGatesR:${a.numGatesR}")
                numGatesR.append(a["numGatesR"])
                numRadialsR.append(a["numRadialsR"])
                timeR.append(a["timeR"])
                rData.append(a["Reflectivity"])

            if len(a["RadialVelocity"]) != 0:
                azimuthV.append(a["azimuthV"])
                elevationV.append(a["elevationV"])
                numGatesV.append(a["numGatesV"])
                numRadialsV.append(a["numRadialsV"])
                timeV.append(a["timeV"])
                vData.append(a["RadialVelocity"])
                sData.append(a["SpectrumWidth"])

            if len(a["SignalQualityIndex"]) != 0:
                SQIData.append(a["SignalQualityIndex"])
            if len(a["TotalReflectivity"]) != 0:
                tData.append(a["TotalReflectivity"])
                ZDRData.append(a["DifferentialReflectivity"])
                CCData.append(a["CrossCorrelationCoefficient"])
                DPData.append(a["DifferentialPhase"])
                KDPData.append(a["SpecificDifferentialPhase"])
                SNRHData.append(a["HorizontalSignalNoiseRatio"])



        if (len(azimuthR) == 11):
            azimuthR = azimuthR[0:1] + azimuthR[2:3] + azimuthR[4:11]
            elevationR = elevationR[0:1] + elevationR[2:3] + elevationR[4:11]
            numGatesR = numGatesR[0:1] + numGatesR[2:3] + numGatesR[4:11]
            numRadialsR = numRadialsR[0:1] + numRadialsR[2:3] + numRadialsR[4:11]
            timeR = timeR[0:1] + timeR[2:3] + timeR[4:11]
            rData = rData[0:1] + rData[2:3] + rData[4:11]
            azimuthV = azimuthV[1:2] + azimuthV[3:11]
            elevationV = elevationV[1:2] + elevationV[3:11]
            numGatesV = numGatesV[1:2] + numGatesV[3:11]
            numRadialsV = numRadialsV[1:2] + numRadialsV[3:11]
            timeV = timeV[1:2] + timeV[3:11]
            vData = vData[1:2] + vData[3:11]
            sData = sData[1:2] + sData[3:11]

        # 选取最大的库数
        print(numGatesR)
        distanceR = list(map(lambda x: x * self.Log_Resolution, range(int(np.max(numGatesR)))))
        distanceV = list(map(lambda x: x * self.Doppler_Resolution, range(int(np.max(numGatesV)))))

        print(numRadialsR, numRadialsV)
        radialMax = max(np.max(np.asarray(numRadialsR)), np.max(np.asarray(numRadialsV)))
        gateRMax = int(np.max(np.asarray(numGatesR)))
        gateVMax = int(np.max(np.asarray(numGatesV)))
        print(gateRMax, gateVMax)
        azimuthR = pad2D(azimuthR, radialMax)
        azimuthV = pad2D(azimuthV, radialMax)
        elevationR = pad2D(elevationR, radialMax)
        elevationV = pad2D(elevationV, radialMax)
        timeR = pad2D(timeR, radialMax)
        timeV = pad2D(timeV, radialMax)
        tData = pad3D(tData, radialMax, gateRMax)
        vData = pad3D(vData, radialMax, gateVMax)
        sData = pad3D(sData, radialMax, gateVMax)
        rData = pad3D(rData, radialMax, gateRMax)

        # print(SQIData)
        if self.isDual:
            SQIData2D = pad3D([SQIData[0]], radialMax, gateRMax)
            ZDRData = pad3D(ZDRData, radialMax, gateRMax)
            CCData = pad3D(CCData, radialMax, gateRMax)
            DPData = pad3D(DPData, radialMax, gateRMax)
            KDPData = pad3D(KDPData, radialMax, gateRMax)
            SNRHData = pad3D(SNRHData, radialMax, gateRMax)
        print(rData.shape)




        b = {"azimuthR": azimuthR, "azimuthV": azimuthV, "distanceR": distanceR, "distanceV": distanceV,
             "elevationR": elevationR, "elevationV": elevationV, "numGatesR": numGatesR, "numGatesV": numGatesV,
             "numRadialsR": numRadialsR, "numRadialsV": numRadialsV, "timeR": timeR, "timeV": timeV,
             "RadialVelocity": vData,
             "Reflectivity": rData, "SpectrumWidth": sData, "TotalReflectivity": tData,
             "SignalQualityIndex": SQIData2D, "DifferentialReflectivity": ZDRData,
             "CrossCorrelationCoefficient": CCData,
             "DifferentialPhase": DPData, "SpecificDifferentialPhase": KDPData, "HorizontalSignalNoiseRatio": SNRHData}
        timtStr=self.datetime.strftime("%Y%m%d%H%M%S")
        outputPath="%s/%s/%s/%s/Z_RADR_I_%s_%s_O_DOR_%s_CAP_FMT.nc"%(outparent,timtStr[:4],timtStr[:8],self.radarID,self.radarID,timtStr,self.type)
        os.makedirs(os.path.dirname(outputPath),exist_ok=True)
        self.mkNC(outputPath, b)
        self.fi.close()
        os.remove(self.path)

if __name__ == "__main__":

    outputFile = "Z_RADR_I_Z9891_20200909000044_O_DOR_CD_CAP_FMT.bin.bz2"
    a = readCinrad(outputFile)
    a.main("./radarNC")




# -*- coding:utf-8 -*-
# Author:hujunnan

import numpy as np

# p = Proj("+proj=lcc +lat_0=30 +lon_0= 102 +lat_1=30 +lat_2=60")

class LAPSTransfrom():
    def __init__(self,p,lonMat, latMat,env=None,step=0.03):
        xMat, yMat = p(lonMat, latMat)
        x0 = np.min(xMat)
        y0 = np.max(yMat)
        if env is None:
            n,s,w,e = np.max(latMat),np.min(latMat),np.min(lonMat),np.max(lonMat)
            lat0 = n
            lat1 = s
            lon0 = w
            lon1 = e
        else:
            lat0 = env.n
            lat1 = env.s
            lon0 = env.w
            lon1 = env.e
        self.step = step
        self.rangeLat = int(np.ceil(((lat0 - lat1) / step))) + 1
        self.rangeLon = int(np.ceil(((lon1 - lon0) / step))) + 1
        self.latArr = np.asarray([lat0 - i*step for i in range(self.rangeLat)],np.float64)
        self.lonArr = np.asarray([lon0 + i*step for i in range(self.rangeLon)],np.float64)
        latMatCHN = np.repeat([self.latArr], self.rangeLon, axis=0).T
        lonMatCHN = np.repeat([self.lonArr], self.rangeLat, axis=0)
        latMatCHNArr = latMatCHN.reshape(-1)
        lonMatCHNArr = lonMatCHN.reshape(-1)
        x, y = p(lonMatCHNArr, latMatCHNArr)
        self.latIdx = ((y0 - y) / (step*100000) + 0.5).astype(int)
        self.lonIdx = ((x - x0) / (step*100000) + 0.5).astype(int)
        self.latMaskValid = np.logical_or(self.latIdx < 0, self.latIdx >= latMat.shape[0])
        self.lonMaskValid = np.logical_or(self.lonIdx < 0, self.lonIdx >= latMat.shape[1])
        self.latIdx[self.latMaskValid] = 0
        self.lonIdx[self.lonMaskValid] = 0

    def LambertToLatLon(self,t):
        lambertMat = t[self.latIdx, self.lonIdx]
        lambertMat = lambertMat.reshape([self.rangeLat, self.rangeLon])
        lambertMat[self.latMaskValid.reshape([self.rangeLat, self.rangeLon])] = np.nan
        lambertMat[self.lonMaskValid.reshape([self.rangeLat, self.rangeLon])] = np.nan
        return lambertMat




# *-* coding=utf8
import numpy as np
import netCDF4 as nc
import datetime
import traceback
from dateutil.relativedelta import relativedelta
import copy
import os

class envelope():
	def __init__(self,n,s,w,e):
		self.n,self.s,self.w,self.e=n,s,w,e
	def __str__(self):
		return ("n:%s,s:%s,w:%s,e:%s"%(self.n,self.s,self.w,self.e))


areaNameDict = {"NEC": "东北","NCN": "华北","CCN": "华中","SCN": "华南","NWC": "西北","SWC": "西南","XJ": "新疆", "XZ": "西藏","CHN":"中国"}        
areaIdxDict = {"NEC": 0,"NCN": 1,"CCN": 2,"SCN": 3,"NWC": 4,"SWC": 5,"XJ": 6, "XZ": 7,"CHN":8}
LeftTopCornerCHN = np.asarray([{"area":"CHN", "evn":envelope(55, 15,72, 136), "CHarea":"中国"}])


LeftTopCornerPairArr = np.asarray([
#   //东北
    {"area":"NEC", "evn":envelope(55, 38,109, 136), "CHarea":"东北"},
#   //华北
    {"area":"NCN", "evn":envelope(43,31, 109,124), "CHarea":"华北"},
#     //华中
    {"area":"CCN", "evn":envelope(36,22, 108, 124), "CHarea":"华中"},
#     //华南
    {"area":"SCN", "evn":envelope(27, 15,104, 124), "CHarea":"华南"},
#     //西北
    {"area":"NWC", "evn":envelope(44,31, 88, 114), "CHarea":"西北"},
#     //西南
    {"area":"SWC", "evn":envelope(35, 20,96, 111), "CHarea":"西南"},
#     //新疆
    {"area":"XJ", "evn":envelope(50, 34,72, 97), "CHarea":"新疆"},
#     //西藏
    {"area":"XZ", "evn":envelope(37,25, 77, 101), "CHarea":"西藏"}])


LeftTopCornerPairArrClip = np.asarray([
#   //东北
    {"area":"NEC", "evn":envelope(55+10e-5, 38+10e-5, 109+10e-5, 136+10e-5), "CHarea":"东北"},
#   //华北
    {"area":"NCN", "evn":envelope(43+10e-5, 31+10e-5, 109+10e-5, 124+10e-5), "CHarea":"华北"},
#     //华中
    {"area":"CCN", "evn":envelope(36+10e-5, 22+10e-5, 108+10e-5, 124+10e-5), "CHarea":"华中"},
#     //华南
    {"area":"SCN", "evn":envelope(27+10e-5, 15+10e-5, 104+10e-5, 124+10e-5), "CHarea":"华南"},
#     //西北
    {"area":"NWC", "evn":envelope(44+10e-5, 31+10e-5, 88+10e-5, 114+10e-5), "CHarea":"西北"},
#     //西南
    {"area":"SWC", "evn":envelope(35+10e-5, 20+10e-5, 96+10e-5, 111+10e-5), "CHarea":"西南"},
#     //新疆
    {"area":"XJ", "evn":envelope(50+10e-5, 34+10e-5, 72+10e-5, 97+10e-5), "CHarea":"新疆"},
#     //西藏
    {"area":"XZ", "evn":envelope(37+10e-5,25+10e-5, 77+10e-5, 101+10e-5), "CHarea":"西藏"}])


LeftTopCornerPairArrSLL = np.asarray([
#   //大渡河
    {"area":"DDH", "evn":envelope(30.96, 28.96,101.21, 103.21)},
#   //贵阳
    {"area":"GY", "evn":envelope(27.08,26.08, 106.23,107.23)},
#     //长春
    {"area":"CC", "evn":envelope(44.4,43.4, 124.72, 125.72)},
#     //深圳
    {"area":"SZ", "evn":envelope(22.86, 22.38,113.76, 114.63)},
#     //天津
    {"area":"TJ", "evn":envelope(40.26,38.56, 116.69, 118.06)},
#     //黄山
    {"area":"HS", "evn":envelope(35, 25,113, 123)},
#     //华山
    {"area":"HUAS", "evn":envelope(36, 30,105, 115)},
#     //武汉
    {"area":"WH", "evn":envelope(31.37, 29.96,113.69, 115.08)},
#     //潍坊
    {"area":"WF", "evn":envelope(37.31, 35.70,118.17,120.0 )},
#     //川藏
    {"area":"CZ", "evn":envelope(33,28, 90, 105)}])
LeftTopCornerPairArrSLLnew = np.asarray([
#   //四川
    {"area":"SC", "evn":envelope(36,25,95,110)},
#     //吉林
    {"area":"JL", "evn":envelope(49,38,121,132)},
#     //广东
    {"area":"GD", "evn":envelope(27,19,109,118)},
#     //京津冀
    {"area":"JJJ", "evn":envelope(44,34,112,121)},
#     //安徽
    {"area":"AH", "evn":envelope(36,29,114,120)},
#     //陕西
    {"area":"SAX", "evn":envelope(41,31,104,113)},
#     //湖北
    {"area":"HB", "evn":envelope(35,28,108,117)},
#     //山东
    {"area":"SD", "evn":envelope(40,33,114,124)}])

class zClass():
    data_ = None
    name_ = None
    type_ = np.float32
    coordinate_ = None
    unit_ = None
    positive_ = None
    def __init__(self, data, name, type_, coordinate, unit,positive):
        self.data_, self.name_, self.type_, self.coordinate_, self.unit_, self.positive_ = data, name, type_, coordinate, unit,positive



class dataClass():
	data_ = None
	name_ = None
	type_ = np.float32
	coordinate_ = None
	unit_ = None
	valid_range_ = [-999,999]
	missing_value_ = np.NAN
	scale_factor_ = np.float32(1.0)
	add_offset_ = np.float32(0.0)

	def __init__(self,data, name, type_,coordinate, unit,missing_value=np.NAN, scale_factor=np.float32(1.0), add_offset=np.float32(0.0),valid_range=[-999,999]):
		self.data_, self.name_, self.type_, self.coordinate_, self.unit_ ,self.missing_value_ ,self.scale_factor_,self.add_offset_,self.valid_range_= data, name,type_, coordinate, unit,missing_value, scale_factor, add_offset, valid_range

	def print(self):
		print(self.data_, self.name_, self.type_, self.coordinate_, self.unit_,self.missing_value_,self.scale_factor_,self.add_offset_,self.valid_range_)
		
def mkNCCommonUni(output,dateTimeStart,dateTimeArr,isoArr,latArr,lonArr,dataClass4D=[],dataClass3D=[],dataClass2D=[],format='NETCDF4'):
    outputTmp = output+".tmp"
    dataset = nc.Dataset(outputTmp,'w',format=format) #'NETCDF4_CLASSIC')
    
    try:
        if not dateTimeArr is None and not dateTimeStart is None: 
            dataset.createDimension("time", len(dateTimeArr))
        if not isoArr is None:
            dataset.createDimension("isobaric", len(isoArr))
        
        dataset.createDimension("lat", len(latArr))
        dataset.createDimension("lon", len(lonArr))
    
        if not dateTimeArr is None and not dateTimeStart is None:
            dataset.createVariable("time", np.float32, ("time"), zlib=True)
        if not isoArr is None:
            dataset.createVariable("isobaric", np.float32, ("isobaric"), zlib=True)
        dataset.createVariable("lat", np.float64, ("lat"), zlib=True)
        dataset.createVariable("lon", np.float64, ("lon"), zlib=True)

        for e in dataClass2D:
            dataset.createVariable(e.name_, e.type_, tuple(["lat","lon"]), zlib=True)    

        for e in dataClass3D:
            dataset.createVariable(e.name_, e.type_, tuple(["time","lat","lon"]), zlib=True)

        for e in dataClass4D:
            dataset.createVariable(e.name_, e.type_, tuple(["time","isobaric","lat","lon"]), zlib=True)
        if not dateTimeArr is None and not dateTimeStart is None:
            dataset.variables["time"][:] = dateTimeArr
            dataset.variables["time"].units = 'minutes since %s'%(dateTimeStart.strftime("%Y-%m-%d %H:%M:%S"))
            dataset.variables["time"].calendar = 'gregorian'

        if not isoArr is None:
            dataset.variables["isobaric"][:] = isoArr
            dataset.variables["isobaric"].units="hPa"
            dataset.variables["isobaric"].positive="up"
        
        dataset.variables["lat"][:] = latArr
        dataset.variables['lat'].units = 'degrees_north'
    
        dataset.variables["lon"][:] = lonArr
        dataset.variables['lon'].units = 'degrees_east'
        
        for e in dataClass2D:
            dataset.variables[e.name_][:] = e.data_
            dataset.variables[e.name_].units = e.unit_
            dataset.variables[e.name_].valid_range = e.valid_range_
            if not "3" in format:
                dataset.variables[e.name_].coordinate = e.coordinate_
            dataset.variables[e.name_].missing_value = e.missing_value_
            dataset.variables[e.name_].scale_factor = e.scale_factor_
            dataset.variables[e.name_].add_offset = e.add_offset_
    
        for e in dataClass3D:
            dataset.variables[e.name_][:] = e.data_
            dataset.variables[e.name_].units = e.unit_
            dataset.variables[e.name_].valid_range = e.valid_range_
            if not "3" in format:
                dataset.variables[e.name_].coordinate = e.coordinate_
            dataset.variables[e.name_].missing_value = e.missing_value_
            dataset.variables[e.name_].scale_factor = e.scale_factor_
            dataset.variables[e.name_].add_offset = e.add_offset_

        for e in dataClass4D:
            dataset.variables[e.name_][:] = e.data_
            dataset.variables[e.name_].units = e.unit_
            dataset.variables[e.name_].valid_range = e.valid_range_
            if not "3" in format:
                dataset.variables[e.name_].coordinate = e.coordinate_
            dataset.variables[e.name_].missing_value = e.missing_value_
            dataset.variables[e.name_].scale_factor = e.scale_factor_
            dataset.variables[e.name_].add_offset = e.add_offset_

        #dataset.close()
    except Exception as ex:
        print(ex)
        print(latArr.shape,lonArr.shape,e.data_.shape)
        print(traceback.format_exc())
        #dataset.close()
    finally:
        dataset.close()
        os.renames(outputTmp,output)

def mkNCCommonUni_multiISO(output, dateTimeStart, dateTimeArr, isoArr, latArr, lonArr, dataClass4D=[], dataClass3D=[],
                  dataClass2D=[], format='NETCDF4'):
    dataset = nc.Dataset(output, 'w', format=format)  # 'NETCDF4_CLASSIC')

    try:
        if not dateTimeArr is None and not dateTimeStart is None:
            dataset.createDimension("time", len(dateTimeArr))


        if not isoArr is None:
            for iso in isoArr:
                dataset.createDimension(iso.name_, len(iso.data_))

        dataset.createDimension("lat", len(latArr))
        dataset.createDimension("lon", len(lonArr))

        if not dateTimeArr is None and not dateTimeStart is None:
            dataset.createVariable("time", np.float32, ("time"), zlib=True)
            
        if not isoArr is None:
            for iso in isoArr:
                dataset.createVariable(iso.name_, iso.type_, iso.coordinate_, zlib=True)

        dataset.createVariable("lat", np.float64, ("lat"), zlib=True)
        dataset.createVariable("lon", np.float64, ("lon"), zlib=True)

        for e in dataClass2D:
            dataset.createVariable(e.name_, e.type_, tuple(e.coordinate_), zlib=True)

        for e in dataClass3D:
            dataset.createVariable(e.name_, e.type_, tuple(e.coordinate_), zlib=True)

        for e in dataClass4D:
            dataset.createVariable(e.name_, e.type_, tuple(e.coordinate_), zlib=True)
        if not dateTimeArr is None and not dateTimeStart is None:
            dataset.variables["time"][:] = dateTimeArr
            dataset.variables["time"].units = 'minutes since %s' % (dateTimeStart.strftime("%Y-%m-%d %H:%M:%S"))
            dataset.variables["time"].calendar = 'gregorian'

        if not isoArr is None:
            for iso in isoArr:
                dataset.variables[iso.name_][:] = iso.data_
                dataset.variables[iso.name_].units = iso.unit_
                dataset.variables[iso.name_].positive = iso.positive_

        dataset.variables["lat"][:] = latArr
        dataset.variables['lat'].units = 'degrees_north'

        dataset.variables["lon"][:] = lonArr
        dataset.variables['lon'].units = 'degrees_east'

        for e in dataClass2D:
            dataset.variables[e.name_][:] = e.data_
            dataset.variables[e.name_].units = e.unit_
            dataset.variables[e.name_].valid_range = e.valid_range_
            if not "3" in format:
                dataset.variables[e.name_].coordinate = e.coordinate_
            dataset.variables[e.name_].missing_value = e.missing_value_
            dataset.variables[e.name_].scale_factor = e.scale_factor_
            dataset.variables[e.name_].add_offset = e.add_offset_

        for e in dataClass3D:
            dataset.variables[e.name_][:] = e.data_
            dataset.variables[e.name_].units = e.unit_
            dataset.variables[e.name_].valid_range = e.valid_range_
            if not "3" in format:
                dataset.variables[e.name_].coordinate = e.coordinate_
            dataset.variables[e.name_].missing_value = e.missing_value_
            dataset.variables[e.name_].scale_factor = e.scale_factor_
            dataset.variables[e.name_].add_offset = e.add_offset_

        for e in dataClass4D:
            dataset.variables[e.name_][:] = e.data_
            dataset.variables[e.name_].units = e.unit_
            dataset.variables[e.name_].valid_range = e.valid_range_
            if not "3" in format:
                dataset.variables[e.name_].coordinate = e.coordinate_
            dataset.variables[e.name_].missing_value = e.missing_value_
            dataset.variables[e.name_].scale_factor = e.scale_factor_
            dataset.variables[e.name_].add_offset = e.add_offset_

        # dataset.close()
    except Exception as ex:
        print(ex)
        print(latArr.shape, lonArr.shape, e.data_.shape)
        print(traceback.format_exc())
        # dataset.close()
    finally:
        dataset.close()


def clip(data, ltc, lat0, lon0, step):
    latIdx0 = int((lat0 - ltc.n) / step+ 0.5)
    latIdx1 = int((lat0 - ltc.s) / step+ 0.5)
    lonIdx0 = int((ltc.w - lon0) / step+ 0.5)
    lonIdx1 = int((ltc.e - lon0) / step+ 0.5)
    data = data[...,latIdx0:latIdx1+1, lonIdx0:lonIdx1+1]
    return data

def clip2(data, ltc, latArr, lonArr, step,isPad=False):
    latIdx0 = int((latArr[0] - ltc.n) / step+ 0.5)
    latIdx1 = int((latArr[0] - ltc.s) / step+ 0.5)
    lonIdx0 = int((ltc.w - lonArr[0]) / step+ 0.5)
    lonIdx1 = int((ltc.e - lonArr[0]) / step+ 0.5)
    if isPad:
        if latArr[latIdx0]<ltc.n and latIdx0>0:
            latIdx0 -=1
        if latArr[latIdx1]>ltc.s and latIdx1< (len(latArr)-1):
            latIdx1 +=1
        if lonArr[lonIdx0] > ltc.w and lonIdx0 > 0:
            lonIdx0 -= 1
        if lonArr[lonIdx1] < ltc.e and lonIdx1 < (len(lonArr) - 1):
            lonIdx1 += 1

    data = data[...,latIdx0:latIdx1+1, lonIdx0:lonIdx1+1]
    return data

def clipLat(data, ltc, step,isPad=False):
    latIdx0 = int((data[0] - ltc.n) / step+ 0.5)
    latIdx1 = int((data[0] - ltc.s) / step+ 0.5)
    if isPad:
        if data[latIdx0]<ltc.n and latIdx0>0:
            latIdx0 -=1
        if data[latIdx1]>ltc.s and latIdx1< (len(data)-1):
            latIdx1 +=1
    data = data[latIdx0:latIdx1+1]
    return data

def clipLon(data, ltc, step,isPad=False):
    lonIdx0 = int((ltc.w - data[0]) / step+ 0.5)
    lonIdx1 = int((ltc.e - data[0]) / step+ 0.5)
    if isPad:
        if data[lonIdx0]>ltc.w and lonIdx0>0:
            lonIdx0 -=1
        if data[lonIdx1]<ltc.e and lonIdx1< (len(data)-1):
            lonIdx1 +=1
    data = data[lonIdx0:lonIdx1+1]
    return data

def totalTimes(delta,second):
    return (delta.days*24*3600+delta.seconds)//second

#def timeSeq(start,end,secInter):
#    times=totalTimes((end-start),secInter)
#    return list(map(lambda x:start+relativedelta(seconds=x*secInter),range(times)))

def timeSeq(start,end,secInter,endPoint=True):
    times=totalTimes((end-start),secInter)
    end = 1 if endPoint else 0
    return list(map(lambda x:start+relativedelta(seconds=x*secInter),range(times+end)))

def UV2WSWD(U,V):
    ws = np.sqrt(np.square(U) + np.square(V))
    wd = (np.degrees(np.arctan2(-U, -V))+ 360)%360
    return ws,wd

def WSWD2UV(ws,wd):
    u=- ws*np.sin(np.radians(wd))
    v=- ws*np.cos(np.radians(wd))
    return u,v

def test():
    print("test")

def normalNC(data):
    mx=np.nanmax(data)
    mn = np.nanmin(data)
    rangeV=mx-mn
    scale = rangeV/254
    offset = (mx+mn)/2
    data1=(data - offset)/scale
    data1[np.isnan(data1)] = -128
    return data1,scale,offset

def SunAngle(UTC, lat, lon, TimeZone = 0):    
    year, DOY, hour = UTC.timetuple().tm_year, UTC.timetuple().tm_yday, UTC.timetuple().tm_hour
    min, second = UTC.timetuple().tm_min, UTC.timetuple().tm_sec
    N0 = 79.6764 + 0.2422 * (year - 1985) - int((year - 1985) / 4.0)
    sitar = 2 * np.pi * (DOY - N0) / 365.2422
    ED1 = 0.3723 + 23.2567 * np.sin(sitar) + 0.1149 * np.sin(2 * sitar) - 0.1712 * np.sin(3 * sitar) - 0.758 * np.cos(sitar) + 0.3656 * np.cos(2 * sitar) + 0.0201 * np.cos(3 * sitar)
    ED = ED1 * np.pi / 180
    dLon = lon - TimeZone * 15.0
    Et = 0.0028 - 1.9857 * np.sin(sitar) + 9.9059 * np.sin(2 * sitar) - 7.0924 * np.cos(
        sitar) - 0.6882 * np.cos(2 * sitar)
    gtdt1 = hour + min / 60.0 + second / 3600.0 + dLon / 15

    gtdt = gtdt1 + Et / 60.0
    dTimeAngle1 = 15.0 * (gtdt - 12)
    dTimeAngle = dTimeAngle1 * np.pi / 180

    latitudeArc = lat * np.pi / 180
    HeightAngleArc = np.arcsin(np.sin(latitudeArc) * np.sin(ED) + np.cos(latitudeArc) * np.cos(ED) * np.cos(dTimeAngle))
    HeightAngle = HeightAngleArc * 180 / np.pi
    return HeightAngle

class clip_and_mosaic():
    def __init__(self,mat,patch,overlap):
        self.mat = mat
        self.patch = patch
        self.overlap = overlap
        center = patch - 2 * overlap
        self.center = center
        self.latRange = int(np.ceil(mat.shape[0]/center))
        lastPaddingLat = self.latRange*center-mat.shape[0]
        self.lonRange = int(np.ceil(mat.shape[1] / center))
        lastPaddingLon = self.lonRange * center - mat.shape[1]
        self.padMat =np.pad(mat,((overlap,lastPaddingLat+overlap),(overlap,lastPaddingLon+overlap)),'constant', constant_values=(0,0))
        self.outMat = np.zeros(self.padMat.shape)

    def clip(self):
        imgList = []
        for i in range(self.latRange):
            for j in range(self.lonRange):
                clipMat = self.padMat[self.center*i:self.center*(i+1)+2*overlap,self.center*j:self.center*(j+1)+2*overlap]
                imgList.append(clipMat)
                # plt.subplot(self.latRange,self.lonRange,j+self.lonRange*i+1)
                # plt.imshow(clipMat)
        return imgList

    def mosaic(self,imgList):
        for i in range(self.latRange):
            for j in range(self.lonRange):
                self.outMat[self.center*i:self.center*(i+1)+2*overlap,self.center*j:self.center*(j+1)+2*overlap] = imgList[j+self.lonRange*i]
        finalMat = self.outMat[overlap:overlap+self.mat.shape[0],overlap:overlap+self.mat.shape[1]]
        return finalMat

# calculate sun angle
def calSunAngle(localTime, TimeZone, lat, lon):
    lon[lon<0] +=360
    year = localTime.year
    month = localTime.month
    day = localTime.day
    hour = localTime.hour
    min = localTime.minute
    sec = localTime.second
    DOY = localTime.timetuple().tm_yday

    N0 = int(79.6764 + 0.2422 * (year - 1985) - ((year - 1985) / 4.0))

    sitar = 2 * np.pi * (DOY - N0) / 365.2422
    ED1 = 0.3723 + 23.2567 * np.sin(sitar) + 0.1149 * np.sin(2 * sitar) - 0.1712 * np.sin(3 * sitar) - 0.758 * np.cos(
        sitar) + 0.3656 * np.cos(2 * sitar) + 0.0201 * np.cos(3 * sitar)
    ED = ED1 * np.pi / 180
    # //    #ED本身有符号

    dLon = copy.copy(lon)

    dLon[np.logical_and(lon >= 0, TimeZone == -13)] = lon[np.logical_and(lon >= 0, TimeZone == -13)] - (
                np.floor((lon[np.logical_and(lon >= 0, TimeZone == -13)] * 10 - 75) / 150) + 1) * 15.0
    dLon[np.logical_and(lon >= 0, TimeZone != -13)] = lon[np.logical_and(lon >= 0, TimeZone != -13)] - TimeZone * 15.0

    dLon[np.logical_and(lon < 0, TimeZone == -13)] = (np.floor(
        (lon[np.logical_and(lon < 0, TimeZone == -13)] * 10 - 75) / 150) + 1) * 15.0 - lon[
                                                         np.logical_and(lon < 0, TimeZone == -13)]
    dLon[np.logical_and(lon < 0, TimeZone != -13)] = TimeZone * 15.0 - lon[np.logical_and(lon < 0, TimeZone != -13)]

    # //    #时差
    Et = 0.0028 - 1.9857 * np.sin(sitar) + 9.9059 * np.sin(2 * sitar) - 7.0924 * np.cos(sitar) - 0.6882 * np.cos(
        2 * sitar)
    gtdt1 = hour + min / 60.0 + sec / 3600.0 + dLon / 15
    # //    #地方时
    gtdt = gtdt1 + Et / 60.0
    dTimeAngle1 = 15.0 * (gtdt - 12)
    dTimeAngle = dTimeAngle1 * np.pi / 180

    latitudeArc = lat * np.pi / 180

    HeightAngleArc = np.arcsin(np.sin(latitudeArc) * np.sin(ED) + np.cos(latitudeArc) * np.cos(ED) * np.cos(dTimeAngle))

    CosAzimuthAngle = (np.sin(HeightAngleArc) * np.sin(latitudeArc) - np.sin(ED)) / np.cos(HeightAngleArc) / np.cos(
        latitudeArc)
    AzimuthAngleArc = np.arccos(CosAzimuthAngle)
    HeightAngle = HeightAngleArc * 180 / np.pi
    ZenithAngle = 90 - HeightAngle
    AzimuthAngle1 = AzimuthAngleArc * 180 / np.pi

    AzimuthAngle = copy.copy(AzimuthAngle1)
    AzimuthAngle[dTimeAngle < 0] = 180 - AzimuthAngle1[dTimeAngle < 0]
    AzimuthAngle[dTimeAngle >= 0] = 180 + AzimuthAngle1[dTimeAngle >= 0]

    # //    println( s"太阳天顶角(deg):${ZenithAngle} 高度角(deg)：${HeightAngle} 方位角(deg)：$AzimuthAngle")
    return ZenithAngle, HeightAngle, AzimuthAngle

def mkDir(path):
    if "." in path:
        os.makedirs(os.path.dirname(path),exist_ok=True)
    else:
        os.makedirs(path, exist_ok=True)


def expend(data,latArr,lonArr,dim=2,fill_value=np.nan,areaLtc=None):
    if areaLtc is None:
        N = LeftTopCornerPairArr[0]["evn"].n
        S = LeftTopCornerPairArr[3]["evn"].s
        W = LeftTopCornerPairArr[6]["evn"].w
        E = LeftTopCornerPairArr[0]["evn"].e
    else:
        N = areaLtc.n
        S = areaLtc.s
        W = areaLtc.w
        E = areaLtc.e

    latOffset = int((N - latArr[0]) / 0.01)
    latOffsetDown = int((latArr[-1] - S) / 0.01)
    lonOffset = int((lonArr[0] - W) / 0.01)
    lonOffsetRight = int((E - lonArr[-1]) / 0.01)
    appendLat = np.asarray(list(map(lambda x: latArr[0] + x * 0.01, range(1, latOffset + 2)))[::-1])
    appendLatDown = np.asarray(list(map(lambda x: latArr[-1] - x * 0.01, range(1, latOffsetDown + 2))))
    latArr = np.r_[appendLat, latArr]
    latArr = np.r_[latArr, appendLatDown]
    appendLon = np.asarray(list(map(lambda x: lonArr[0] - x * 0.01, range(1, lonOffset + 2)))[::-1])
    appendLonRight = np.asarray(list(map(lambda x: lonArr[-1] + x * 0.01, range(1, lonOffsetRight + 2))))
    lonArr = np.r_[appendLon, lonArr]
    lonArr = np.r_[lonArr, appendLonRight]

    if latOffset<0:
        latOffset=0
    if latOffsetDown < 0:
        latOffsetDown = 0
    if lonOffset < 0:
        lonOffset = 0
    if lonOffsetRight < 0:
        lonOffsetRight = 0

    if dim==2:
        data = np.pad(data, ((latOffset+1 , latOffsetDown+1 ), (lonOffset+1 , lonOffsetRight+1)),constant_values=fill_value)
    elif dim==3:
        data = np.pad(data, ((0, 0), (latOffset+1, latOffsetDown+1), (lonOffset+1, lonOffsetRight+1)), constant_values=fill_value)
    elif dim==4:
        data = np.pad(data, ((0, 0),(0, 0), (latOffset+1, latOffsetDown+1), (lonOffset+1, lonOffsetRight+1)), constant_values=fill_value)
    return data,latArr,lonArr


def checkNC(path):
    with nc.Dataset(path) as dataNC:
        for e in list(dataNC.variables):
            dataNC[e][:]

def cropDF(df,evn):
    return df[np.logical_and.reduce([df["Lat"]>evn.s,df["Lat"]<evn.n,df["Lon"]>evn.w,df["Lon"]<evn.e])]





import hashlib
import time
def DAA_example(userId,pwd,params):
    t = time.time()
    para = f"serviceNodeId=NMIC_MUSIC_CMADAAS&userId={userId}&{params}"
    timeStep = "timestamp=%s000"%int(t)
    noncemd5 = hashlib.md5()
    noncemd5.update(timeStep.encode('utf-8'))
    nonce="nonce=%s"%(noncemd5.hexdigest().upper())
    pwd = f"pwd={pwd}"
    paraSK = "&".join([timeStep,nonce,pwd])
    allPara = para+"&"+paraSK
    list= allPara.split("&")
    list.sort()
    paramSort = "&".join(list)
    md5 = hashlib.md5()
    md5.update(paramSort.encode('utf-8'))
    sign ="sign="+md5.hexdigest().upper()
    prefix = "http://10.40.17.54/music-ws/api?"
    url = prefix + para + "&"+"&".join([timeStep,nonce,sign])
    return url

if __name__ == '__main__':
    print(DAA_example("a","b","c=1&d=1"))





# Author : bb
# Time : 2020/11/18 16:59
# -*- coding: utf-8 -*-

import numpy as np
from sklearn.neighbors import KDTree
np.seterr(divide='ignore',invalid='ignore')

class IDW():
    def __init__(self, y, x,   candidate=15):
        self.disMat = None
        self.idMat = None
        self.disPoint = None
        self.idPoint = None
        self.x = x
        self.y = y
        self.resultMat = None
        self.resultPont = None
        self.candidate = candidate

    def initDictMat(self,ltc,resolution=0.01):
        self.evn = ltc
        self.resolution = resolution
        self.latRange = int((self.evn.n - self.evn.s + 10e-10) // self.resolution) + 1
        self.lonRange = int((self.evn.e - self.evn.w + 10e-10) // self.resolution) + 1
        self.latArr = np.linspace(self.evn.n, self.evn.s, self.latRange)
        self.lonArr = np.linspace(self.evn.w, self.evn.e, self.lonRange)
        latArr = np.linspace(self.evn.n, self.evn.s, self.latRange)
        lonArr = np.linspace(self.evn.w, self.evn.e, self.lonRange)
        lonMat, latMat = np.meshgrid(lonArr, latArr)
        tree = KDTree(np.c_[self.y, self.x])
        xyMesh = np.c_[latMat.reshape(-1), lonMat.reshape(-1)]
        dist, ind = tree.query(xyMesh, k=self.candidate)
        self.disMat = dist.reshape([len(latArr), len(lonArr), self.candidate])
        self.idMat = ind.reshape([len(latArr), len(lonArr), self.candidate])

    def getMat(self,z):
        valMat = z[self.idMat]
        disAll = np.sum(1 / np.power(self.disMat, 2), axis=2)
        dataAll = np.sum(valMat / np.power(self.disMat, 2), axis=2)
        self.resultMat = dataAll / disAll
        self.resultMat[self.disMat[:, :, 0] == 0] = valMat[self.disMat[:, :, 0] == 0, 0]
        return self.resultMat

    def initDictPoints(self,yP, xP):
        tree = KDTree(np.c_[self.y, self.x])
        xyMesh = np.c_[yP, xP]
        dist, ind = tree.query(xyMesh, k=self.candidate)
        self.disPoint = dist
        self.idPoint = ind
        print()

    def getPoint(self, z):
        valPont = z[self.idPoint]
        disAll = np.sum(1 / np.power(self.disPoint, 2),axis=1)
        dataAll = np.sum(valPont / np.power(self.disPoint, 2),axis=1)
        self.resultPont = dataAll / disAll
        self.resultPont[self.disPoint[:,0] == 0] = valPont[self.disPoint[:,0] == 0, 0]
        return self.resultPont












from PIL import Image, ImageDraw
import matplotlib.pyplot as plt
from hjn.mkNCHJN import envelope,mkDir
import numpy as np
import netCDF4 as nc
import copy
import geopandas as gpd
from osgeo import gdal, ogr, osr
import os

class maskClip():
    def __init__(self,data,latAtt,lonArr,step):

        self.dem = data
        self.latArr = latAtt
        self.lonArr = lonArr

        oriStep=np.abs((self.latArr[0]-self.latArr[-1])/(len(self.latArr)-1))
        range=int(np.round(step/oriStep,0))
        self.dem=self.dem[::range,::range]

        self.ltc = envelope(self.latArr[0], self.latArr[-1], self.lonArr[0], self.lonArr[-1])
        self.step = step

    def world2Pixel(self,ltc, x, y):
        """
        Uses a gdal geomatrix (gdal.GetGeoTransform()) to calculate
        the pixel location of a geospatial coordinate
        """
        ulX = ltc.w
        ulY = ltc.n
        xDist = self.step

        pixel = int((x - ulX) / xDist)
        line = int((ulY - y) / xDist)
        return (pixel, line)


    def getMask(self,geom,minX,maxY,minY,maxX):

        points = []
        pixels = []
        ulX, ulY = self.world2Pixel(self.ltc, minX, maxY)
        lrX, lrY = self.world2Pixel(self.ltc, maxX, minY)

        # Calculate the pixel size of the new image
        pxWidth = int(lrX - ulX)
        pxHeight = int(lrY - ulY)

        ltc1 = copy.copy(self.ltc)
        ltc1.n = maxY
        ltc1.w = minX

        pts = geom.boundary.xy
        for p in range(len(pts[0])):
            points.append((pts[0][p], pts[1][p]))
        for p in points:
            pixels.append(self.world2Pixel(ltc1, p[0], p[1]))
        rasterPoly = Image.new("L", (pxWidth, pxHeight), 1)
        rasterize = ImageDraw.Draw(rasterPoly)
        if len(pixels) > 1:
            rasterize.polygon(pixels, 0)

        mask = np.asarray(rasterPoly)


        # plt.imshow(mask)
        # plt.show()

        latOffset0 = int((self.ltc.n - maxY) / self.step)
        latOffset1 = self.dem.shape[0] - int((self.ltc.n - minY) / self.step)
        lonOffset0 = int((minX - self.ltc.w) / self.step)
        lonOffset1 = self.dem.shape[1] - int((maxX - self.ltc.w) / self.step)
        ndarray = np.pad(mask, ((latOffset0, latOffset1),
                                (lonOffset0, lonOffset1)), 'constant', constant_values=(1, 1))

        clip = copy.copy(self.dem)

        clip[ndarray != 0] = np.nan

        return clip



    def clip(self,shapefile_path,encoding='gb18030'):
        chly = gpd.GeoDataFrame.from_file(shapefile_path, encoding=encoding).geometry

        minX, maxX, minY, maxY = chly.bounds.min().minx,chly.bounds.max().maxx,chly.bounds.min().miny,chly.bounds.max().maxy
        maskArr=[]

        for j in range(len(chly)):
            geom = chly[j]
            if geom.geometryType() == 'Polygon':
                clip=self.getMask(geom, minX, maxY, minY, maxX)
                maskArr.append(clip)
            elif geom.geometryType() == "MultiPolygon":
                multiLayer = []
                for i in range(len(geom)):
                    geom1 = geom[i]
                    clip = self.getMask(geom1, minX, maxY, minY, maxX)
                    multiLayer.append(clip)

                multiLayer= np.asarray(multiLayer)
                clip = np.nanmax(multiLayer,axis=0)
                maskArr.append(clip)

        return maskArr

    def getMaskMask(self, geom, minX, maxY, minY, maxX):

        points = []
        pixels = []
        ulX, ulY = self.world2Pixel(self.ltc, minX, maxY)
        lrX, lrY = self.world2Pixel(self.ltc, maxX, minY)

        # Calculate the pixel size of the new image
        pxWidth = int(lrX - ulX)
        pxHeight = int(lrY - ulY)

        ltc1 = copy.copy(self.ltc)
        ltc1.n = maxY
        ltc1.w = minX

        pt = geom.boundary
        # print(pt)

        if pt.type == "LineString":
            pts = pt.xy
            for p in range(len(pts[0])):
                points.append((pts[0][p], pts[1][p]))
            for p in points:
                pixels.append(self.world2Pixel(ltc1, p[0], p[1]))
            rasterPoly = Image.new("L", (pxWidth, pxHeight), 1)
            rasterize = ImageDraw.Draw(rasterPoly)
            if len(pixels) > 1:
                rasterize.polygon(pixels, 0)

            mask = np.asarray(rasterPoly)

            latOffset0 = int((self.ltc.n - maxY) / self.step)
            latOffset1 = self.dem.shape[0] - int((self.ltc.n - minY) / self.step)
            lonOffset0 = int((minX - self.ltc.w) / self.step)
            lonOffset1 = self.dem.shape[1] - int((maxX - self.ltc.w) / self.step)
            ndarray = np.pad(mask, ((latOffset0, latOffset1),
                                    (lonOffset0, lonOffset1)), 'constant', constant_values=(1, 1))

            # clip = copy.copy(self.dem)
            # clip[ndarray != 0] = np.nan
            return mask
        else:
            # return
            masks = []

            for ptt in pt[:1]:
                pts = ptt.xy
                for p in range(len(pts[0])):
                    points.append((pts[0][p], pts[1][p]))
                for p in points:
                    pixels.append(self.world2Pixel(ltc1, p[0], p[1]))
                rasterPoly = Image.new("L", (pxWidth, pxHeight), 1)
                rasterize = ImageDraw.Draw(rasterPoly)
                if len(pixels) > 1:
                    rasterize.polygon(pixels, 0)

                mask = np.asarray(rasterPoly)
                masks.append(mask)
            masks = np.asarray(masks)
            masks = np.min(masks, axis=0)
            latOffset0 = int((self.ltc.n - maxY) / self.step)
            latOffset1 = self.dem.shape[0] - int((self.ltc.n - minY) / self.step)
            lonOffset0 = int((minX - self.ltc.w) / self.step)
            lonOffset1 = self.dem.shape[1] - int((maxX - self.ltc.w) / self.step)
            ndarray = np.pad(masks, ((latOffset0, latOffset1),
                                     (lonOffset0, lonOffset1)), 'constant', constant_values=(1, 1))

            clip = copy.copy(self.dem)
            clip[ndarray != 0] = np.nan
            return clip

    def clipMask(self,chlyTable):
        # chlyTable = gpd.GeoDataFrame.from_file(shapefile_path, encoding=encoding)
        chly = chlyTable.geometry
        minX, maxX, minY, maxY = chly.bounds.min().minx,chly.bounds.max().maxx,chly.bounds.min().miny,chly.bounds.max().maxy
        maskArr=[]

        for j in range(len(chly)):
            geom = chly.iloc[j]
            if geom.geometryType() == 'Polygon':
                clip=self.getMaskMask(geom, minX, maxY, minY, maxX)
                if not clip is None:
                    maskArr.append(clip)

            elif geom.geometryType() == "MultiPolygon":
                multiLayer = []
                for i in range(len(geom)):
                    geom1 = geom[i]
                    clip = self.getMaskMask(geom1, minX, maxY, minY, maxX)
                    multiLayer.append(clip)

                multiLayer= np.asarray(multiLayer)
                clip = np.nanmin(multiLayer,axis=0)
                if not clip is None:
                    maskArr.append(clip)

        return maskArr


class slope_aspect():
    def __init__(self,gridNew,dx, dy):
        self.gridNew = gridNew
        self.dx = dx
        self.dy = dy
        self.Sx, self.Sy = self.calcFiniteSlopes(gridNew, self.dx, self.dy)
    # 计算dx,dy
    def calcFiniteSlopes(self,elevGrid, dx, dy):
        Zbc = np.pad(elevGrid, ((1, 1), (1, 1)), "edge")
        Sx = (Zbc[1:-1, :-2] - Zbc[1:-1, 2:]) / (2 * dx)  # WE方向
        Sy = (Zbc[2:, 1:-1] - Zbc[:-2, 1:-1]) / (2 * dy)  # NS方向
        return Sx, Sy

    def calcFiniteAspect(self,Sx, Sy):

        Sx = Sx.astype(np.float)
        Sy = Sy.astype(np.float)
        aspect = np.degrees(np.arctan2(Sy, Sx))
        aspect1 = copy.copy(aspect)
        aspect1[aspect < 0] = 90 - aspect[aspect < 0]
        aspect1[aspect > 90] = 360.0 - aspect[aspect > 90] + 90.0
        aspect1[np.logical_and(aspect <= 90, aspect >= 0)] = 90.0 - aspect[np.logical_and(aspect <= 90, aspect >= 0)]
        aspect1[np.logical_and(Sx == 0, Sy == 0)] = -1
        return aspect1

    def getSlop(self):
        return np.degrees(np.arctan(np.sqrt(self.Sx ** 2 + self.Sy ** 2)))

    def getAspect(self):
        return self.calcFiniteAspect(self.Sx, self.Sy)


def Polygonize(mask,latArr,lonArr,outshp,EPSG=4326):
    driver = gdal.GetDriverByName("GTiff")
    im_bands = 1
    dataset = driver.Create("./tmp.tiff", mask.shape[1], mask.shape[0], im_bands, gdal.GDT_UInt16)
    im_geotrans = (lonArr[0], lonArr[1]-lonArr[0], 0, latArr[0], 0, latArr[1]-latArr[0])
    dataset.SetGeoTransform(im_geotrans)
    prj = osr.SpatialReference()
    prj.ImportFromEPSG(EPSG)
    dataset.SetProjection(prj.ExportToWkt())
    dataset.GetRasterBand(1).WriteArray(mask) 
    inband = dataset.GetRasterBand(1)
    mkDir(outshp)
    drv = ogr.GetDriverByName("ESRI Shapefile")
    if os.path.exists(outshp):
        drv.DeleteDataSource(outshp)
    Polygon = drv.CreateDataSource(outshp)
    Poly_layer = Polygon.CreateLayer(outshp, srs=prj,
                                     geom_type=ogr.wkbMultiPolygon)
    newField = ogr.FieldDefn('value', ogr.OFTReal)
    Poly_layer.CreateField(newField)
    gdal.FPolygonize(inband, None, Poly_layer, 0)
    Polygon.SyncToDisk()
    del Polygon
    os.remove("./tmp.tiff")





import numpy as np
import copy
import os
import datetime

def bilinear(y,  x, yul, xul, vul,yur,  xur, vur, ydl, xdl, vdl, ydr,  xdr,vdr):
    u = linear(x, xul, vul, xur, vur)
    yu = linear(x, xul, yul, xur, yur)
    b = linear(x, xdl, vdl, xdr, vdr)
    yb = linear(x, xdl, ydl, xdr, ydr)
    v = linear(y, yu, u, yb, b)
    return v


def linear(x, xl, vl0, xr, vr0):
    vl=vl0
    vr=vr0
    if vl == vr:
        v=vl
    else:
        dxl = x - xl
        drl = xr - xl
        v = dxl * (vr - vl) / drl + vl
    return v

def getPoint3D(pre,df,lat0,lon0,resolution,decimal=1):
   
    latIdx = ((lat0 - df["Lat"]) / resolution + 0.5).astype(np.int64)
    lonIdx = ((df["Lon"] - lon0) / resolution + 0.5).astype(np.int64)
    return pre[:, latIdx, lonIdx].round(decimals=decimal)


def getPoint2D(pre, df, lat0, lon0, resolution, decimal=1):
    latIdx = ((lat0 - df["Lat"]) / resolution + 0.5).astype(np.int64)
    lonIdx = ((df["Lon"] - lon0) / resolution + 0.5).astype(np.int64)
    return pre[latIdx, lonIdx].round(decimals=decimal)

def getPoint(pre, df, lat0, lon0, resolution, decimal=1):
    latIdx = ((lat0 - df["Lat"]) / resolution + 0.5).astype(np.int64)
    lonIdx = ((df["Lon"] - lon0) / resolution + 0.5).astype(np.int64)
    return pre[...,latIdx, lonIdx].round(decimals=decimal)

def getPointBilinear3D(preData,df1,latArr,lonArr,decimal=2):
    df=copy.copy(df1)
    resolution=np.abs((latArr[0]-latArr[-1])/(len(latArr)-1))
    latIdxN = ((latArr[0] - df["Lat"]) / resolution).astype(np.int64)
    lonIdxW = ((df["Lon"] - lonArr[0]) / resolution).astype(np.int64)

    latIdxS = ((latArr[0] - df["Lat"]) / resolution).astype(np.int64)+1
    lonIdxE = ((df["Lon"] - lonArr[0]) / resolution).astype(np.int64)+1

    df["LatN"] = latArr[latIdxN]
    df["LonW"] = lonArr[lonIdxW]
    df["valNW"] = preData[:, latIdxN, lonIdxW]

    df["LatS"] = latArr[latIdxS]
    df["LonE"] = lonArr[lonIdxE]
    df["valSE"] = preData[:, latIdxS, lonIdxE]
    df["valNE"] = preData[:, latIdxN, lonIdxE]
    df["valSW"] = preData[:, latIdxS, lonIdxW]
    return df.apply(
                lambda x:np.round(bilinear(x["Lat"], x["Lon"], x["LatN"], x["LonW"], x["valNW"], x["LatN"], x["LonE"], x["valNE"], x["LatS"],
                                   x["LonW"], x["valSW"], x["LatS"], x["LonE"], x["valSE"]),decimals=decimal), axis=1)


def getPointBilinear2D(preData,df1,latArr,lonArr,decimal=2):
    df=copy.copy(df1)
    resolution=np.abs((latArr[0]-latArr[-1])/(len(latArr)-1))
    latIdxN = ((latArr[0] - df["Lat"]) / resolution).astype(np.int64)
    lonIdxW = ((df["Lon"] - lonArr[0]) / resolution).astype(np.int64)

    latIdxS = ((latArr[0] - df["Lat"]) / resolution).astype(np.int64)+1
    lonIdxE = ((df["Lon"] - lonArr[0]) / resolution).astype(np.int64)+1

    df["LatN"] = latArr[latIdxN]
    df["LonW"] = lonArr[lonIdxW]
    df["valNW"] = preData[latIdxN, lonIdxW]

    df["LatS"] = latArr[latIdxS]
    df["LonE"] = lonArr[lonIdxE]
    df["valSE"] = preData[ latIdxS, lonIdxE]
    df["valNE"] = preData[ latIdxN, lonIdxE]
    df["valSW"] = preData[ latIdxS, lonIdxW]
    return df.apply(
                lambda x:np.round(bilinear(x["Lat"], x["Lon"], x["LatN"], x["LonW"], x["valNW"], x["LatN"], x["LonE"], x["valNE"], x["LatS"],
                                   x["LonW"], x["valSW"], x["LatS"], x["LonE"], x["valSE"]),decimals=decimal), axis=1)
def getPointBilinear(preData,df1,latArr,lonArr,decimal=2):
    df=copy.copy(df1)
    resolution=np.abs((latArr[0]-latArr[-1])/(len(latArr)-1))
    latIdxN = ((latArr[0] - df["Lat"]) / resolution).astype(np.int64)
    lonIdxW = ((df["Lon"] - lonArr[0]) / resolution).astype(np.int64)

    latIdxS = ((latArr[0] - df["Lat"]) / resolution).astype(np.int64)+1
    lonIdxE = ((df["Lon"] - lonArr[0]) / resolution).astype(np.int64)+1

    df["LatN"] = latArr[latIdxN]
    df["LonW"] = lonArr[lonIdxW]
    df["valNW"] = preData[..., latIdxN, lonIdxW]

    df["LatS"] = latArr[latIdxS]
    df["LonE"] = lonArr[lonIdxE]
    df["valSE"] = preData[..., latIdxS, lonIdxE]
    df["valNE"] = preData[..., latIdxN, lonIdxE]
    df["valSW"] = preData[..., latIdxS, lonIdxW]
    return df.apply(
                lambda x:np.round(bilinear(x["Lat"], x["Lon"], x["LatN"], x["LonW"], x["valNW"], x["LatN"], x["LonE"], x["valNE"], x["LatS"],
                                   x["LonW"], x["valSW"], x["LatS"], x["LonE"], x["valSE"]),decimals=decimal), axis=1)



def classify10min(pre0):
    pre = copy.deepcopy(pre0)
    pre[pre0 < 0.1] = 0
    pre[np.logical_and(pre0 >= 0.1, pre0 <= 0.5)] = 1
    pre[np.logical_and(pre0 > 0.5, pre0 <= 1)] = 2
    pre[np.logical_and(pre0 > 1, pre0 <= 2)] = 3
    pre[np.logical_and(pre0 > 2, pre0 <= 9990)] = 4
    pre[pre0 > 9990] = -1
    pre[np.isnan(pre0)] = -1
    return pre

def classify30min(pre0):
    pre = copy.deepcopy(pre0)
    pre[pre0 < 0.1] = 0
    pre[np.logical_and(pre0 >= 0.1, pre0 <= 2)] = 1
    pre[np.logical_and(pre0 > 2, pre0 <= 4)] = 2
    pre[np.logical_and(pre0 > 4, pre0 <= 10)] = 3
    pre[np.logical_and(pre0 > 10, pre0 <= 9990)] = 4
    pre[pre0 > 9990] = -1
    pre[np.isnan(pre0)] = -1
    return pre

def classify1h(pre0):
    pre = copy.deepcopy(pre0)
    pre[pre0 < 0.1] = 0
    pre[np.logical_and(pre0 >= 0.1, pre0 <= 2.5)] = 1
    pre[np.logical_and(pre0 > 2.5, pre0 <= 8)] = 2
    pre[np.logical_and(pre0 > 8, pre0 <= 16)] = 3
    pre[np.logical_and(pre0 > 16, pre0 <= 9990)] = 4
    pre[pre0 > 9990] = -1
    pre[np.isnan(pre0)] = -1
    return pre

# classify 3h
def classify3h(pre0):
    pre = copy.deepcopy(pre0)
    pre[pre0 < 0.1] = 0
    pre[np.logical_and(pre0 >= 0.1, pre0 <= 3)] = 1
    pre[np.logical_and(pre0 > 3, pre0 <= 10)] = 2
    pre[np.logical_and(pre0 > 10, pre0 <= 20)] = 3
    pre[np.logical_and(pre0 > 20, pre0 <= 9990)] = 4
    pre[pre0 > 9990] = -1
    pre[np.isnan(pre0)] = -1
    return pre

def degMinSectoDeg(a):
    deg = int(a.split("°")[0])
    sec = int(a.split("'")[-1][:-1])
    min = int(a.replace(f"{deg}°","").replace(f"'{sec}\"",""))
    return deg+min/60+sec/3600

def normal(data, min=None, max=None):
    if max is None:
        max = np.max(data)
    if min is None:
        min = np.min(data)
    rangeLat = max-min
    dataB = (data - min) / rangeLat
    return dataB

def mkDir(path):
    if "." in path:
        os.makedirs(os.path.dirname(path),exist_ok=True)
    else:
        os.makedirs(path, exist_ok=True)

def options():
    import argparse
    parser = argparse.ArgumentParser(description='hjn')
    parser.add_argument('--times', type=str, default='2018060700,2018060700')
    parser.add_argument('--isDebug',action='store_true',default=False)
    config= parser.parse_args()
    config.times = config.times.split(",")
    if len(config.times) == 1:
        config.times = [config.times[0], config.times[0]]
    config.times = [datetime.datetime.strptime(config.times[0], "%Y%m%d%H%M"),
                    datetime.datetime.strptime(config.times[1], "%Y%m%d%H%M")]

    return config

class Evaluate():
    def __init__(self,num_class):
        self.num_class = num_class
        self.confusion_matrix = np.zeros((self.num_class,)*2)

    def Mean_Intersection_over_Union(self):
        MIoU = np.diag(self.confusion_matrix) / (
                np.sum(self.confusion_matrix, axis=1) + np.sum(self.confusion_matrix, axis=0) -
                np.diag(self.confusion_matrix))
        MIoU = np.nanmean(MIoU)
        return MIoU

    def _generate_matrix(self, gt_image, pre_image):
        mask = (gt_image >= 0) & (gt_image < self.num_class)
        label = self.num_class * gt_image[mask].astype('int') + pre_image[mask]
        count = np.bincount(label, minlength=self.num_class**2)
        confusion_matrix = count.reshape(self.num_class, self.num_class)
        return confusion_matrix

    def add_batch(self, gt_image, pre_image):
        assert gt_image.shape == pre_image.shape
        self.confusion_matrix += self._generate_matrix(gt_image, pre_image)



import numpy as np
from hjn.mkNCHJN import envelope

class FY4Proj():
    def __init__(self,lonD=104.7,resolution=4000):

        self.ea=6378.137
        self.eb=6356.7523
        self.h=42164
        self.lambdaD=np.radians(lonD)        

        OFF={500:10991.5,1000:5495.5,2000:2747.5,4000:1373.5}
        FAC={500:81865099,1000:40932549,2000:20466274,4000:10233137}       
        
        self.COFF=OFF[resolution]
        self.LOFF=OFF[resolution]
        self.CFAC=FAC[resolution]
        self.LFAC=FAC[resolution]

    def transform(self,latD,lonDe):
        lat=np.radians(latD)
        lon=np.radians(lonDe)
        ba2=np.square(self.eb/self.ea)
        phie=np.arctan(ba2*np.tan(lat))
        diffLon0=lon-self.lambdaD
        re=self.eb/np.sqrt(1-(1-ba2)*np.square(np.cos(phie)))

        r1=self.h-re*np.cos(phie)*np.cos(diffLon0)
        r2= -re*np.cos(phie)*np.sin(diffLon0)
        r3=re*np.sin(phie)
        rn=np.sqrt(np.square(r1)+np.square(r2)+np.square(r3))


        x= np.degrees(np.arctan(-r2/r1))
        y= np.degrees(np.arcsin(-r3/rn))

        c=(self.COFF+x*np.power(2.0,-16)*self.CFAC -0.5).astype(np.int32)
        l=(self.LOFF+y*np.power(2.0,-16)*self.LFAC -0.5).astype(np.int32)
        return (l,c)

    def transform_inver(self,l,c):
        eab2 = (self.ea / self.eb) ** 2
        x = np.radians((c - self.COFF) / (np.power(2.0, -16) * self.CFAC))
        y = np.radians((l - self.LOFF) / (np.power(2.0, -16) * self.LFAC))
        sd = np.sqrt((self.h*np.cos(x)*np.cos(y))**2-(np.cos(y)**2+eab2*np.sin(y)**2)*(self.h**2-self.ea**2))
        sn = (self.h*np.cos(x)*np.cos(y)-sd)/(np.cos(y)**2+eab2*np.sin(y)**2)
        s1 = self.h-sn*np.cos(x)*np.cos(y)
        s2 = sn*np.sin(x)*np.cos(y)
        s3 = -sn*np.sin(y)
        sxy = np.sqrt(s1**2+s2**2)
        lon = np.degrees(np.arctan(s2/s1))+np.degrees(self.lambdaD)

        lon[lon>180] -=360
        lat = np.degrees(np.arctan(eab2*s3 / sxy))
        return (lat,lon)
    


    def transforMat(self,ltc,step,SNCmat):
        _,_,latMat,lonMat = getLatlonMat(ltc,step)
        (l,c)=self.transform(latMat,lonMat)
        snclatlon=SNCmat[l,c]
        return snclatlon

class H8Proj():
    def __init__(self, lonD=140.7, resolution=2000):
        self.ea = 6378.137
        self.eb = 6356.7523
        self.h = 42165.32745491
        self.lambdaD = np.radians(lonD)

        OFF = {500: 11000.5, 1000: 5500.5 , 2000: 2750.5}
        FAC = {500: 81865099, 1000: 40932549, 2000: 20466275}

        self.COFF = OFF[resolution]
        self.LOFF = OFF[resolution]
        self.CFAC = FAC[resolution]
        self.LFAC = FAC[resolution]


    def transform(self,latD, lonDe):
        lat = np.radians(latD)
        lon = np.radians(lonDe)
        ba2 = np.square(self.eb / self.ea)
        phie = np.arctan(ba2 * np.tan(lat))
        diffLon0 = lon - self.lambdaD
        re = self.eb / np.sqrt(1 - (1 - ba2) * np.square(np.cos(phie)))

        r1 = self.h - re * np.cos(phie) * np.cos(diffLon0)
        r2 = -re * np.cos(phie) * np.sin(diffLon0)
        r3 = re * np.sin(phie)
        rn = np.sqrt(np.square(r1) + np.square(r2) + np.square(r3))

        x = np.degrees(np.arctan(-r2 / r1))
        y = np.degrees(np.arcsin(-r3 / rn))

        c = (self.COFF + x * np.power(float(2), -16) * self.CFAC - 0.5).astype(np.int32)
        l = (self.LOFF + y * np.power(float(2), -16) * self.LFAC - 0.5).astype(np.int32)
        return (l, c)

    def transforMat(self,ltc,step,SNCmat,loff):
        _,_,latMat,lonMat = getLatlonMat(ltc,step)
        (l,c)=self.transform(latMat,lonMat)
        snclatlon=SNCmat[l-int(loff),c]
        return snclatlon


def getLatlonMat(ltc,step,endPoint=True):
    end = 1 if endPoint else 0
    latArr = np.linspace(ltc.n, ltc.s, int((ltc.n - ltc.s) / step) + end)
    lonArr = np.linspace(ltc.w, ltc.e, int((ltc.e - ltc.w) / step) + end)
    latMat = np.dot(latArr.reshape(-1, 1), np.ones([1, len(lonArr)]))
    lonMat = np.dot(np.ones([len(latArr), 1]), np.expand_dims(lonArr, axis=0))
    return latArr,lonArr,latMat,lonMat



import numpy as np
import os
import datetime
import argparse
from hjn.mkNCHJN import LeftTopCornerPairArr
import time
from dateutil.relativedelta import relativedelta
import hashlib

def degMinSectoDeg(a):
    deg = int(a.split("°")[0])
    sec = int(a.split("'")[-1][:-1])
    min = int(a.replace(f"{deg}°","").replace(f"'{sec}\"",""))
    return deg+min/60+sec/3600

def normal(data, min=None, max=None):
    if max is None:
        max = np.max(data)
    if min is None:
        min = np.min(data)
    rangeLat = max-min
    dataB = (data - min) / rangeLat
    return dataB

def mkDir(path):
    if "." in path:
        os.makedirs(os.path.dirname(path),exist_ok=True)
    else:
        os.makedirs(path, exist_ok=True)

def options():
    parser = argparse.ArgumentParser(description='hjn')
    parser.add_argument('--times', type=str, default='2018060700,2018060700')
    parser.add_argument('--isDebug',action='store_true',default=False)
    config= parser.parse_args()
    config.times = config.times.split(",")
    if len(config.times) == 1:
        config.times = [config.times[0], config.times[0]]
    config.times = [datetime.datetime.strptime(config.times[0], "%Y%m%d%H%M"),
                    datetime.datetime.strptime(config.times[1], "%Y%m%d%H%M")]

    return config


def totalTimes(delta,second):
    return (delta.days*24*3600+delta.seconds)//second

def timeSeq(start,end,secInter,endPoint=True):
    times=totalTimes((end-start),secInter)
    end = 1 if endPoint else 0
    return list(map(lambda x:start+relativedelta(seconds=x*secInter),range(times+end)))

def normalNC(data):
    mx=np.nanmax(data)
    mn = np.nanmin(data)
    rangeV=mx-mn
    scale = rangeV/254
    offset = (mx+mn)/2
    data1=(data - offset)/scale
    data1[np.isnan(data1)] = -128
    return data1,scale,offset


def expend(data,latArr,lonArr,dim=2):
    latOffset = int((LeftTopCornerPairArr[0]["evn"].n - latArr[0]) / 0.01)
    latOffsetDown = int((latArr[-1] - LeftTopCornerPairArr[3]["evn"].s) / 0.01)
    lonOffset = int((lonArr[0] - LeftTopCornerPairArr[6]["evn"].w) / 0.01)
    lonOffsetRight = int((LeftTopCornerPairArr[0]["evn"].e - lonArr[-1]) / 0.01)
    appendLat = np.asarray(list(map(lambda x: latArr[0] + x * 0.01, range(1, latOffset + 2)))[::-1])
    appendLatDown = np.asarray(list(map(lambda x: latArr[-1] - x * 0.01, range(1, latOffsetDown + 2))))
    latArr = np.r_[appendLat, latArr]
    latArr = np.r_[latArr, appendLatDown]
    appendLon = np.asarray(list(map(lambda x: lonArr[0] - x * 0.01, range(1, lonOffset + 2)))[::-1])
    appendLonRight = np.asarray(list(map(lambda x: lonArr[-1] + x * 0.01, range(1, lonOffsetRight + 2))))
    lonArr = np.r_[appendLon, lonArr]
    lonArr = np.r_[lonArr, appendLonRight]

    if latOffset<0:
        latOffset=0
    if latOffsetDown < 0:
        latOffsetDown = 0
    if lonOffset < 0:
        lonOffset = 0
    if lonOffsetRight < 0:
        lonOffsetRight = 0

    if dim==2:
        data = np.pad(data, ((latOffset+1 , latOffsetDown+1 ), (lonOffset+1 , lonOffsetRight+1)),constant_values=np.nan)
    elif dim==3:
        data = np.pad(data, ((0, 0), (latOffset+1, latOffsetDown+1), (lonOffset+1, lonOffsetRight+1)), constant_values=np.nan)
    elif dim==4:
        data = np.pad(data, ((0, 0),(0, 0), (latOffset+1, latOffsetDown+1), (lonOffset+1, lonOffsetRight+1)), constant_values=np.nan)
    return data,latArr,lonArr

def UV2WSWD(U,V):
    ws = np.sqrt(np.square(U) + np.square(V))
    wd = (np.degrees(np.arctan2(-U, -V))+ 360)%360
    return ws,wd

def WSWD2UV(ws,wd):
    u=- ws*np.sin(np.radians(wd))
    v=- ws*np.cos(np.radians(wd))
    return u,v

def findAllIPV4(segment="10."):
    import psutil
    from socket import AddressFamily
    local_addrs = ""
    for name, info in psutil.net_if_addrs().items():
        for addr in info:
            if AddressFamily.AF_INET == addr.family:
                if segment in addr.address:
                    local_addrs = addr.address
    return local_addrs

def sendEmail(message,subject = "fail"):
    import requests
    IP = findAllIPV4()
    response = None
    try:
        response = requests.get("http://10.16.50.218:5004/sendmail?address=408037825@qq.com&subject=%s:%s&message=%s"%(subject,IP,message))
    except Exception as e:
        print(e)
    return response

def seconds2datetime(seconds):
    return datetime.datetime.strptime(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(seconds)), "%Y-%m-%d %H:%M:%S")

def datetime2seconds(UTC):
    return time.mktime(UTC.timetuple())

def readJulianDateTime(juliandata, hms):
    jd = juliandata + 2440587
    L = jd + 68569
    N = (4 * L) // 146097
    L = L - (146097 * N + 3) // 4
    I = 4000 * (L + 1) // 1461001
    L = L - (1461 * I) // 4 + 31
    J = 80 * L // 2447
    day = L - 2447 * J // 80
    L = int(J // 11)
    month = J + 2 - 12 * L
    year = 100 * (N - 49) + I + L
    seconds = hms // 1000
    h = seconds // 3600
    m = (seconds - h * 3600) // 60
    s = seconds - (60 * h + m) * 60
    return datetime.datetime(year, month, day, h, m, s)

def RGB_to_Hex(tmp):
    rgb = tmp.split(',')  # 将RGB格式划分开来
    strs = '#'
    for i in rgb:
        num = int(i)  # 将str转int
        # 将R、G、B分别转化为16进制拼接转换并大写
        strs += str(hex(num))[-2:].replace('x', '0').upper()

    return strs

def genLatLon(evn,resolution):
    latArr = np.linspace(evn.n,evn.s,int(((evn.n-evn.s)/resolution+1)))
    lonArr = np.linspace(evn.w,evn.e,int(((evn.e-evn.w)/resolution+1)))
    return latArr,lonArr


def calSHA1(path):
    encrypt = hashlib.sha1()
    with open(path,"rb") as f:
        while True:
            b = f.read(128000)
            encrypt.update(b)
            if not b:
                break
    sha1Result = encrypt.hexdigest()
    return sha1Result




from gpuinfo import GPUInfo
import numpy as np
import time

def findGPU(waitS=2):
    """get ID of GPUS
    :param num_gpu:  num of GPUs to use
    :return: gpu_id: ID of allocated GPUs
    """
    gpu_id = None
    waitSec = np.random.uniform(0,waitS)
    time.sleep(waitSec)
    available_device=GPUInfo.check_empty()
    if not available_device is None:
        if len(available_device)>0:
            idx = np.random.randint(0,len(available_device))
            gpu_id=available_device[idx]

    return gpu_id





import os
from pyinotify import WatchManager, Notifier,ProcessEvent,IN_DELETE, IN_CREATE,IN_MODIFY


class EventHandler(ProcessEvent):
    """事件处理"""
    def process_IN_CREATE(self, event):
        print("Create file: %s " %  os.path.join(event.path,event.name))
        
    def process_IN_DELETE(self, event):
        print("Delete file: %s " %  os.path.join(event.path,event.name))
 
    def process_IN_MODIFY(self, event):
        filePath=os.path.join(event.path,event.name)
        if "tweet" in event.name:
            print("Modify file: %s " % filePath )
            print("*"*30)
            try:
                with  open(filePath , 'r') as f:
                    print (f.read())
            except Exception as e:
                print(e)

def FSMonitor(path='.'):
    wm = WatchManager() 
    mask = IN_DELETE | IN_CREATE |IN_MODIFY
    notifier = Notifier(wm, EventHandler())
    wm.add_watch(path, mask,auto_add=True,rec=True)
    print('now starting monitor %s'%(path))
    while True:
        try:
            notifier.process_events()
            if notifier.check_events():
                notifier.read_events()
        except KeyboardInterrupt:
            notifier.stop()
            break

if __name__ == "__main__":
    FSMonitor('/var/log/shiny-server')




import numpy as np
from hjn.mkNCHJN import envelope
import datetime

class parseHimawari():
    def __init__(self):
        self.res = 12100000
        self.nlin = 1100
        self.ncol = 11000
        self.data=None

    def read_Himawari(self,inputfile):
        resolution=int(inputfile[-12:-10])
        if resolution==10:
            self.res=12100000
            self.nlin=1100
            self.ncol=11000
        elif resolution==20:
            self.res=3025000
            self.nlin=550
            self.ncol=5500
        elif resolution == 5:
            self.res=48400000
            self.nlin=2200
            self.ncol=22000
        band=int(inputfile[-21:-19])
        if band < 7:
            formation = [('Block number1', 'i1', 1), \
                         ('Block length1', 'i2', 1), \
                         ('Total number of header blocks ', 'i2', 1), \
                         ('Byte order', 'i1', 1), \
                         ('Satellite name', 'S1', 16), \
                         ('Processing center name', 'S1', 16), \
                         ('Observation area', 'S1', 4), \
                         ('Other observation information', 'S1', 2), \
                         ('Observation timeline', 'i2', 1), \
                         ('Observation start time', 'float64', 1), \
                         ('Observation end time', 'float64', 1), \
                         ('File creation time', 'float64', 1), \
                         ('Total header length', 'i4', 1), \
                         ('Total data length', 'i4', 1), \
                         ('Quality flag 1', 'i1', 1), \
                         ('Quality flag 2 ', 'i1', 1), \
                         ('Quality flag 3', 'i1', 1), \
                         ('Quality flag 4', 'i1', 1), \
                         ('File format version', 'S1', 32), \
                         ('File name ', 'S1', 128), \
                         ('Spare1', 'S1', 40), \

                         ('Block number2', 'i1', 1), \
                         ('Block length2', 'i2', 1), \
                         ('Number of bits per pixel', 'i2', 1), \
                         ('Number of columns', 'i2', 1), \
                         ('Number of lines', 'i2', 1), \
                         ('Compression flag for data', 'i1', 1), \
                         ('Spare2', 'S1', 40), \

                         ('Block number3', 'i1', 1), \
                         ('Block length3', 'i2', 1), \
                         ('sub_lon', 'float64', 1), \
                         ('Column scaling factor', 'i4', 1), \
                         ('Line scaling factor', 'i4', 1), \
                         ('Column offset', 'float32', 1), \
                         ('Line offset', 'float32', 1), \
                         ('Distance from Earth’s center to virtual satellite', 'float64', 1), \
                         ('Earth’s equatorial radius', 'float64', 1), \
                         ('Earth’s polar radius', 'float64', 1), \
                         ('var1', 'float64', 1), \
                         ('var2', 'float64', 1), \
                         ('var3', 'float64', 1), \
                         ('Coefficient for sd', 'float64', 1), \
                         ('Resampling types', 'i2', 1), \
                         ('Resampling size', 'i2', 1), \
                         ('Spare3', 'S1', 40), \

                         ('Block number4', 'i1', 1), \
                         ('Block length4', 'i2', 1), \
                         ('Navigation information time', 'float64', 1), \
                         ('SSP longitude', 'float64', 1), \
                         ('SSP latitude', 'float64', 1), \
                         ('Distance from Earth’s center to Satellite', 'float64', 1), \
                         ('Nadir longitude', 'float64', 1), \
                         ('Nadir latitude', 'float64', 1), \
                         ('Sun’s position', 'float64', 3), \
                         ('Moon’s position', 'float64', 3), \
                         ('Spare4', 'S1', 40), \

                         ('Block number5', 'i1', 1), \
                         ('Block length5', 'i2', 1), \
                         ('Band number', 'i2', 1), \
                         ('Central wave length', 'float64', 1), \
                         ('Valid number of bits per pixel', 'i2', 1), \
                         ('Count value of error pixels', 'uint16', 1), \
                         ('Count value of pixels outside scan area', 'uint16', 1), \
                         ('Slope for count-radiance conversion equation', 'float64', 1), \
                         ('Intercept for count-radiance conversion equation', 'float64', 1), \
                         ('Coefficient for transformation from radiance  to albedo', 'float64', 1), \
                         ('Update time of the values of the following No. 12 and No. 13', 'float64', 1), \
                         ('Calibrated Slope for count-radiance conversion equation_updated value of No. 8 of this block ', 'float64', 1),\
                         ('Calibrated Intercept for count-radiance conversion equation_updated value of No. 9 of this block ', 'float64', 1),\
                         ('Spare5', 'S1', 80), \

                         ('Block number6', 'i1', 1), \
                         ('Block length6', 'i2', 1), \
                         ('GSICS calibration coefficient_Intercept', 'float64', 1), \
                         ('GSICS calibration coefficient_Slope', 'float64', 1), \
                         ('GSICS calibration coefficient_Quadratic term', 'float64', 1), \
                         ('Radiance bias for standard scene', 'float64', 1), \
                         ('Uncertainty of radiance bias for standard scene', 'float64', 1), \
                         ('Radiance for standard scene', 'float64', 1), \
                         ('Start time of GSICS Correction validity period', 'float64', 1), \
                         ('End time of GSICS Correction validity period', 'float64', 1), \
                         ('Radiance validity range of GSICS calibration coefficients_upper limit', 'float32', 1), \
                         ('Radiance validity range of GSICS calibration coefficients_lower limit', 'float32', 1), \
                         ('File name of GSICS Correction', 'S1', 128), \
                         ('Spare6', 'S1', 56), \

                         ('Block number7', 'i1', 1), \
                         ('Block length7', 'i2', 1), \
                         ('Total number of segments', 'i1', 1), \
                         ('Segment sequence number', 'i1', 1), \
                         ('First line number of image segment', 'i2', 1), \
                         ('Spare7', 'S1', 40), \

                         ('Block number8', 'i1', 1), \
                         ('Block length8', 'i2', 1), \
                         ('Center column of rotation', 'float32', 1), \
                         ('Center line of rotation', 'float32', 1), \
                         ('Amount of rotational correction', 'float64', 1), \
                         ('Number of correction information data for column and line direction', 'i2', 1), \
                         ('Line number after rotation', 'i2', 1), \
                         ('Shift amount for column direction', 'float32', 1), \
                         ('Shift amount for line direction8', 'float32', 1), \
                         ('Spare8', 'S1', 50), \

                         ('Block number9', 'i1', 1), \
                         ('Block length9', 'i2', 1), \
                         ('Number of observation times9', 'i2', 1), \
                         ('Line number9', 'i2', 1), \
                         ('Observation time9', 'float64', 1), \
                         ('Spare9', 'S1', 70), \

                         ('Block number10', 'i1', 1), \
                         ('Block length10', 'i4', 1), \
                         ('Number of error information data', 'i2', 1), \
                         ('Line number10', 'i2', 1), \
                         ('Number of error pixels per line10', 'i2', 1), \
                         ('Spare10', 'S1', 36), \

                         ('Block number11', 'i1', 1), \
                         ('Block length11', 'i2', 1), \
                         ('Spare11', 'S1', 256), \

                         ('Count value of each pixel', 'i2', self.res)]
        else:
            formation=[('Block number1','i1',1),\
                        ('Block length1','i2',1),\
                        ('Total number of header blocks ','i2',1),\
                        ('Byte order','i1',1),\
                        ('Satellite name','S1',16),\
                        ('Processing center name','S1',16),\
                        ('Observation area','S1',4),\
                        ('Other observation information','S1',2),\
                        ('Observation timeline','i2',1),\
                        ('Observation start time','float64',1),\
                        ('Observation end time','float64',1),\
                        ('File creation time','float64',1),\
                        ('Total header length','i4',1),\
                        ('Total data length','i4',1),\
                        ('Quality flag 1','i1',1),\
                        ('Quality flag 2 ','i1',1),\
                        ('Quality flag 3','i1',1),\
                        ('Quality flag 4','i1',1),\
                        ('File format version','S1',32),\
                        ('File name ','S1',128),\
                        ('Spare1','S1',40),\

                        ('Block number2','i1',1),\
                        ('Block length2','i2',1),\
                        ('Number of bits per pixel','i2',1),\
                        ('Number of columns','i2',1),\
                        ('Number of lines','i2',1),\
                        ('Compression flag for data','i1',1),\
                        ('Spare2','S1',40),\

                        ('Block number3','i1',1),\
                        ('Block length3','i2',1),\
                        ('sub_lon','float64',1),\
                        ('Column scaling factor','i4',1),\
                        ('Line scaling factor','i4',1),\
                        ('Column offset','float32',1),\
                        ('Line offset','float32',1),\
                        ('Distance from Earth’s center to virtual satellite','float64',1),\
                        ('Earth’s equatorial radius','float64',1),\
                        ('Earth’s polar radius','float64',1),\
                        ('var1','float64',1),\
                        ('var2','float64',1),\
                        ('var3','float64',1),\
                        ('Coefficient for sd','float64',1),\
                        ('Resampling types','i2',1),\
                        ('Resampling size','i2',1),\
                        ('Spare3','S1',40),\

                        ('Block number4','i1',1),\
                        ('Block length4','i2',1),\
                        ('Navigation information time','float64',1),\
                        ('SSP longitude','float64',1),\
                        ('SSP latitude','float64',1),\
                        ('Distance from Earth’s center to Satellite','float64',1),\
                        ('Nadir longitude','float64',1),\
                        ('Nadir latitude','float64',1),\
                        ('Sun’s position','float64',3),\
                        ('Moon’s position','float64',3),\
                        ('Spare4','S1',40),\

                        ('Block number5','i1',1),\
                        ('Block length5','i2',1),\
                        ('Band number','i2',1),\
                        ('Central wave length','float64',1),\
                        ('Valid number of bits per pixel','i2',1),\
                        ('Count value of error pixels','i2',1),\
                        ('Count value of pixels outside scan area','i2',1),\
                        ('Slope for count-radiance conversion equation','float64',1),\
                        ('Intercept for count-radiance conversion equation','float64',1),\
                        ('radiance to brightness temperature_c0','float64',1),\
                        ('radiance to brightness temperature_c1','float64',1),\
                        ('radiance to brightness temperature_c2','float64',1),\
                        ('brightness temperature to radiance_C0','float64',1),\
                        ('brightness temperature to radianceC1','float64',1),\
                        ('brightness temperature to radianceC2','float64',1),\
                        ('Speed of light','float64',1),\
                        ('Planck constant','float64',1),\
                        ('Boltzmann constant','float64',1),\
                        ('Spare5','S1',40), \

                       ('Block number6', 'i1', 1), \
                       ('Block length6', 'i2', 1), \
                       ('GSICS calibration coefficient_Intercept', 'float64', 1), \
                       ('GSICS calibration coefficient_Slope', 'float64', 1), \
                       ('GSICS calibration coefficient_Quadratic term', 'float64', 1), \
                       ('Radiance bias for standard scene', 'float64', 1), \
                       ('Uncertainty of radiance bias for standard scene', 'float64', 1), \
                       ('Radiance for standard scene', 'float64', 1), \
                       ('Start time of GSICS Correction validity period', 'float64', 1), \
                       ('End time of GSICS Correction validity period', 'float64', 1), \
                       ('Radiance validity range of GSICS calibration coefficients_upper limit', 'float32', 1), \
                       ('Radiance validity range of GSICS calibration coefficients_lower limit', 'float32', 1), \
                       ('File name of GSICS Correction', 'S1', 128), \
                       ('Spare6', 'S1', 56), \

                       ('Block number7', 'i1', 1), \
                       ('Block length7', 'i2', 1), \
                       ('Total number of segments', 'i1', 1), \
                       ('Segment sequence number', 'i1', 1), \
                       ('First line number of image segment', 'i2', 1), \
                       ('Spare7', 'S1', 40), \

                       ('Block number8', 'i1', 1), \
                       ('Block length8', 'i2', 1), \
                       ('Center column of rotation', 'float32', 1), \
                       ('Center line of rotation', 'float32', 1), \
                       ('Amount of rotational correction', 'float64', 1), \
                       ('Number of correction information data for column and line direction', 'i2', 1), \
                       ('Line number after rotation', 'i2', 1), \
                       ('Shift amount for column direction', 'float32', 1), \
                       ('Shift amount for line direction8', 'float32', 1), \
                       ('Spare8', 'S1', 50), \

                       ('Block number9', 'i1', 1), \
                       ('Block length9', 'i2', 1), \
                       ('Number of observation times9', 'i2', 1), \
                       ('Line number9', 'i2', 1), \
                       ('Observation time9', 'float64', 1), \
                       ('Spare9', 'S1', 70), \

                       ('Block number10', 'i1', 1), \
                       ('Block length10', 'i4', 1), \
                       ('Number of error information data', 'i2', 1), \
                       ('Line number10', 'i2', 1), \
                       ('Number of error pixels per line10', 'i2', 1), \
                       ('Spare10', 'S1', 36), \

                       ('Block number11', 'i1', 1), \
                       ('Block length11', 'i2', 1), \
                       ('Spare11', 'S1', 256), \


                       ('Count value of each pixel', 'i2', self.res)]

        self.data=np.fromfile(inputfile,dtype=formation)[0]
        band=int(inputfile[-21:-19])

        mat = self.data["Count value of each pixel"].reshape([self.nlin, self.ncol])
        mat=self.toRealVal(mat,band)
        return mat

    def toRealVal(self,mat,bandNo):
        radiance1=mat * self.data["Slope for count-radiance conversion equation"] + self.data["Intercept for count-radiance conversion equation"]

        radiance2 = radiance1*self.data["Coefficient for transformation from radiance  to albedo"] if bandNo >= 1 and bandNo <= 6 else self.hisd_radiance_to_tbb(radiance1)

        return radiance2

    def hisd_radiance_to_tbb(self, radiance0):
        lmbd = self.data['Central wave length'] / 1000000.0
        radiance = radiance0 * 1000000.0
        planck_c1 = 2.0 * self.data['Planck constant'] * self.data['Speed of light']**2.0 / lmbd**5.0
        planck_c2 = self.data['Planck constant'] * self.data['Speed of light'] / (self.data['Boltzmann constant'] * lmbd)
        effective_temperature = planck_c2 / np.log((planck_c1 / radiance) + 1.0)
        tbb = self.data['radiance to brightness temperature_c0'] + self.data['radiance to brightness temperature_c1'] * effective_temperature + self.data['radiance to brightness temperature_c2'] * effective_temperature**2
        tbb[radiance<0]=np.nan
        return tbb

if __name__=="__main__":
    h8P=parseHimawari()
    data = h8P.read_Himawari("")

